from copy import deepcopy
from typing import Dict, List, Union

import numpy as np

from freemocap.data_layer.skeleton_models.segments import Segment
from freemocap.data_layer.skeleton_models.skeleton import Skeleton


def calculate_bone_lengths_and_statistics(
        marker_data: Dict[str, np.ndarray], segment_connections: Dict[str, Segment]
) -> Dict[str, Dict[str, Union[np.ndarray, float]]]:
    """
    Calculates bone lengths for each frame and their statistics (median and standard deviation)
    based on marker positions and segment connections.

    Parameters:
    - marker_data: A dictionary containing marker trajectories with marker names as keys and
      3D positions as values (numpy arrays).
    - segment_connections: A dictionary defining the segments (bones) with segment names as keys
      and dictionaries with 'proximal' and 'distal' markers as values.

    Returns:
    - A dictionary with segment names as keys and dictionaries with lengths, median lengths,
      and standard deviations as values.
    """
    bone_statistics = {}

    for segment_name, segment in segment_connections.items():
        proximal_positions = marker_data[segment.proximal]
        distal_positions = marker_data[segment.distal]

        lengths = np.linalg.norm(distal_positions - proximal_positions, axis=1)
        valid_lengths = lengths[~np.isnan(lengths)]

        median_length = np.median(valid_lengths)
        stdev_length = np.std(valid_lengths)

        bone_statistics[segment_name] = {"lengths": lengths, "median": median_length, "stdev": stdev_length}

    return bone_statistics


def enforce_rigid_bones(
        marker_data: Dict[str, np.ndarray],
        segment_connections: Dict[str, Segment],
        bone_lengths_and_statistics: Dict[str, Dict[str, Union[np.ndarray, float]]],
        joint_hierarchy: Dict[str, List[str]],
) -> Dict[str, np.ndarray]:
    """
    Enforces rigid bones by adjusting the distal joints of each segment to match the median length.

    Parameters:
    - marker_data: The original marker positions.
    - segment_connections: Information about how segments (bones) are connected.
    - bone_lengths_and_statistics: The desired bone lengths and statistics for each segment.
    - joint_hierarchy: The hierarchy of joints, indicating parent-child relationships.

    Returns:
    - A dictionary of adjusted marker positions.
    """
    rigid_marker_data = deepcopy(marker_data)

    for segment_name, stats in bone_lengths_and_statistics.items():
        desired_length = stats["median"]
        lengths = stats["lengths"]

        segment = segment_connections[segment_name]
        proximal_marker, distal_marker = segment.proximal, segment.distal

        for frame_index, current_length in enumerate(lengths):
            if current_length != desired_length:
                proximal_position = marker_data[proximal_marker][frame_index]
                distal_position = marker_data[distal_marker][frame_index]
                direction = distal_position - proximal_position
                try:
                    direction /= np.linalg.norm(direction)  # Normalize to unit vector
                except ZeroDivisionError:
                    direction /= 1e-5  # Set to a small value if the direction is zero
                adjustment = (desired_length - current_length) * direction

                rigid_marker_data[distal_marker][frame_index] += adjustment

                adjust_children(distal_marker, frame_index, adjustment, rigid_marker_data, joint_hierarchy)

    return rigid_marker_data


def adjust_children(
        parent_marker: str,
        frame_index: int,
        adjustment: np.ndarray,
        marker_data: Dict[str, np.ndarray],
        joint_hierarchy: Dict[str, List[str]],
):
    """
    Recursively adjusts the positions of child markers based on the adjustment of the parent marker.
    """
    if parent_marker in joint_hierarchy:
        for child_marker in joint_hierarchy[parent_marker]:
            marker_data[child_marker][frame_index] += adjustment
            adjust_children(child_marker, frame_index, adjustment, marker_data, joint_hierarchy)


def merge_rigid_marker_data(rigid_marker_data: Dict[str, np.ndarray]) -> np.ndarray:
    """
    Merges the center of mass data from multiple segments into a single array.

    Parameters:
    - segment_com_data: A dictionary where each key is a segment name and the value is the center of mass data for that segment.

    Returns:
    - A numpy array containing the merged center of mass data.
    """
    # TODO: We could use this more broadly as a skeleton method

    rigid_marker_data_list = list(rigid_marker_data.values())

    return np.stack(rigid_marker_data_list, axis=1)


def enforce_rigid_bones_from_skeleton(skeleton: Skeleton) -> np.ndarray:
    """
    Calculates bone lengths and statistics from skeleton data and enforces rigid bones.

    Parameters:
    - skeleton: The Skeleton instance containing segment information, joint hierarchy, and marker data.

    Returns:
    - A numpy array of adjusted marker positions.
    """
    # TODO: Should this be a method of Skeleton?
    if not skeleton.segments:
        raise ValueError("Segments must be defined before rigid bones can be enforced.")

    if not skeleton.joint_hierarchy:
        raise ValueError("Joint hierarchy must be defined before rigid bones can be enforced.")

    bone_lengths_and_statistcs = calculate_bone_lengths_and_statistics(
        marker_data=skeleton.marker_data, segment_connections=skeleton.segments
    )

    rigid_marker_data = enforce_rigid_bones(
        marker_data=skeleton.marker_data,
        segment_connections=skeleton.segments,
        bone_lengths_and_statistics=bone_lengths_and_statistcs,
        joint_hierarchy=skeleton.joint_hierarchy,
    )

    return merge_rigid_marker_data(rigid_marker_data=rigid_marker_data)



import subprocess
import sys
from pathlib import Path

import pandas as pd

# repo root = three levels up from this script
repo_root = Path(__file__).resolve().parents[3]
print("🧭 repo_root =", repo_root)

summary_csv = repo_root / "freemocap/diagnostics/diagnostic_data/calibration_diagnostics_summary.csv"
collected = Path("collected")  # where download-artifact puts the CSVs
# 1) load existing summary
if summary_csv.exists():
    full_df = pd.read_csv(summary_csv)
    print(f"Loaded existing summary with {len(full_df)} rows")
else:
    # Create empty dataframe with expected columns if file doesn't exist
    full_df = pd.DataFrame(columns=["os", "version", "mean_distance", "median_distance", "std_distance", "mean_error"])
    print("Created new summary dataframe")

# 2) ingest rows
csv_files = list(collected.glob("**/*.csv"))
print(f"Found {len(csv_files)} CSV files to merge:")
for f in csv_files:
    print(f"  - {f}")

if not csv_files:
    sys.exit("❌ No calibration rows found in ./collected")

rows = []
for f in csv_files:
    try:
        df = pd.read_csv(f)
        print(f"\n  Reading {f.name}:")
        print(f"    Shape: {df.shape}")
        print(f"    Columns: {list(df.columns)}")
        if len(df) > 0:
            print("    First row:")
            print(f"      OS: '{df.iloc[0]['os']}'")
            print(f"      Version: '{df.iloc[0]['version']}'")
            print(f"      Mean distance: {df.iloc[0]['mean_distance']}")
        rows.append(df)
    except Exception as e:
        print(f"  Error reading {f}: {e}")
        # Try to read raw content
        with open(f, "r") as fh:
            print(f"  Raw content of {f.name}:")
            print(fh.read())

if not rows:
    sys.exit("❌ No valid CSV data found")

new_df = pd.concat(rows, ignore_index=True)
print(f"\nCombined into {len(new_df)} new rows")
print("New data preview:")
print(new_df.to_string())

# Standardize OS names
new_df["os"] = new_df["os"].str.strip()

# 3) replace old 'current' rows and save
full_df = full_df[full_df["version"] != "current"]
full_df = pd.concat([full_df, new_df], ignore_index=True)

# Ensure OS names are standardized
full_df["os"] = full_df["os"].str.strip()

# Remove duplicates - keep only the latest entry for each os/version combination
print(f"Before deduplication: {len(full_df)} rows")
full_df = full_df.drop_duplicates(subset=["os", "version"], keep="last")
print(f"After deduplication: {len(full_df)} rows")

print(f"Final dataframe has {len(full_df)} rows")
print(f"OS values: {full_df['os'].unique()}")
print(f"Version values: {full_df['version'].unique()}")

# Show duplicate check
duplicates = full_df[full_df.duplicated(subset=["os", "version"], keep=False)]
if len(duplicates) > 0:
    print(f"WARNING: Still have {len(duplicates)} duplicate rows!")
    print(duplicates[["os", "version"]].value_counts())

summary_csv.parent.mkdir(parents=True, exist_ok=True)
full_df.to_csv(summary_csv, index=False)
print(f"✅ Summary updated: {summary_csv}")

# 4) regenerate HTML
report_script = repo_root / "freemocap/diagnostics/calibration/generate_calibration_report.py"
if report_script.exists():
    print(f"Running report generation script: {report_script}")
    result = subprocess.run([sys.executable, str(report_script)], capture_output=True, text=True)
    print("STDOUT:", result.stdout)
    if result.stderr:
        print("STDERR:", result.stderr)
    if result.returncode != 0:
        sys.exit(f"Report generation failed with code {result.returncode}")
else:
    print(f"⚠️ Report script not found at {report_script}")

print("🎉 Process complete!")



import logging
import platform
from pathlib import Path
from typing import Union, Optional

logger = logging.getLogger(__name__)


def guess_blender_exe_path_from_path(base_path: Union[str, Path]) -> Optional[Path]:
    base_path = Path(base_path)
    try:
        blender_folder_paths = [path for path in base_path.rglob("blender.exe")]
    except OSError:
        logger.info(f"Unable to access: {str(base_path)}")
        return

    if blender_folder_paths:

        if len(blender_folder_paths) == 0:
            return None

        best_guess = blender_folder_paths[-1]

        return best_guess


def get_best_guess_of_blender_path():
    if platform.system() == "Windows":
        # check all lettered drives and the user's home directory
        paths_to_check = [
            Path(f"{letter}:/Program Files/Blender Foundation") for letter in "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        ]

        paths_to_check.append(Path.home() / "Blender Foundation")

        for base_path in paths_to_check:
            blender_exe_path = guess_blender_exe_path_from_path(base_path)
            if blender_exe_path is not None and blender_exe_path.is_file():
                logger.info(f"Found `blender.exe` at: {str(blender_exe_path)}")

                return str(blender_exe_path)
        else:
            logger.warning(
                "Could not find `blender.exe` in the expected locations. Please locate it manually (or install Blender, if it isn't installed)."
            )
            return None

    if platform.system() == "Darwin":
        blender_app_path = Path("/Applications/Blender.app")

        if blender_app_path.exists():
            logger.info(f"Mac machine detected - guessing that `blender` is installed at: {str(blender_app_path)}")

            blender_exe_path = blender_app_path / "Contents/MacOS/Blender"
            return str(blender_exe_path)
        else:
            logger.warning(
                "Could not find Blender executable in the applications folder. Please locate it manually (or install Blender, if it isn't installed)."
            )
            return None

    if platform.system() == "Linux":
        blender_path_list = [
            Path("/usr/bin"),
            Path("/usr/sbin"),
            Path("/usr/local/bin"),
            Path("/usr/local/sbin"),
            Path("/bin"),
            Path("/sbin"),
            Path("/snap/bin"),
        ]
        for path in blender_path_list:
            blender_path = path / "blender"
            if blender_path.exists():
                logger.info(f"Linux machine detected - guessing that `blender` is installed at: {str(blender_path)}")

                return str(blender_path)

        logger.info(
            "Could not find Blender executable in bin. Please locate it manually (or install Blender, if it isn't installed)."
        )
        return None

    else:
        logger.info("Machine system not detected, please locate Blender path manually.")
        return None


if __name__ == "__main__":
    print(f" blender path: {get_best_guess_of_blender_path()}")



from typing import Optional

from PySide6.QtCore import Qt, QObject
from PySide6.QtGui import QDesktopServices, QCursor
from PySide6.QtWidgets import QLabel

CALIBRATION_DOCS_LINK = "https://freemocap.github.io/documentation/multi-camera-calibration.html"


class CalibrationGuideLinkQLabel(QLabel):
    """A reusable widget that displays a clickable link to the calibration documentation."""

    def __init__(
            self,
            text: str | None = None,
            url: str | None = None,
            parent: Optional[QObject] = None,
    ):
        """
        Initialize the CalibrationGuideLink widget.

        Args:
            text: The link text to display. Defaults to "📚 Calibration Guide"
            url: The URL to open. Defaults to CALIBRATION_DOCS_LINK
            parent: The parent widget
        """
        super().__init__(parent=parent)

        self._url = url or CALIBRATION_DOCS_LINK
        link_text = text or "📚 Calibration Guide"

        # Set up the link appearance and behavior
        self.setText(
            f'<a href="{self._url}" style="color: #4a90e2; text-decoration: none;">{link_text}</a>'
        )
        self.setOpenExternalLinks(False)  # Handle the click ourselves for better control
        self.setCursor(QCursor(Qt.CursorShape.PointingHandCursor))
        self.setToolTip("View calibration documentation and best practices")

        # Connect the link activation to the handler
        self.linkActivated.connect(self._open_calibration_docs)

    def _open_calibration_docs(self, url: str) -> None:
        """Open the calibration documentation in the default web browser."""
        QDesktopServices.openUrl(url=url)

    def set_url(self, url: str) -> None:
        """
        Update the URL that the link points to.

        Args:
            url: The new URL to use
        """
        self._url = url
        # Update the href in the label text
        current_text = self.text()
        # Extract the link text (everything between > and </a>)
        import re
        match = re.search(r'>(.*?)</a>', current_text)
        if match:
            link_text = match.group(1)
            self.setText(
                f'<a href="{self._url}" style="color: #4a90e2; text-decoration: none;">{link_text}</a>'
            )



import sys

from PySide6.QtCore import QDir
from PySide6.QtGui import QFileSystemModel, QAction
from PySide6.QtWidgets import QTreeView, QFileIconProvider, QScroller, QApplication, QMenu


class DirectoryViewer(QTreeView):
    def __init__(self, root_path=None, no_custom=False, no_watch=False, parent=None):
        super().__init__(parent)

        self.model = QFileSystemModel()
        icon_provider = QFileIconProvider()
        self.model.setIconProvider(icon_provider)
        self.model.setRootPath("")
        if no_custom:
            self.model.setOption(QFileSystemModel.DontUseCustomDirectoryIcons)
        if no_watch:
            self.model.setOption(QFileSystemModel.DontWatchForChanges)
        self.setModel(self.model)
        if root_path:
            root_index = self.model.index(QDir.cleanPath(root_path))
            if root_index.isValid():
                self.setRootIndex(root_index)

        # Demonstrating look and feel features
        self.setAnimated(False)
        self.setIndentation(20)
        self.setSortingEnabled(True)
        availableSize = self.screen().availableGeometry().size()
        self.resize(availableSize / 2)
        self.setColumnWidth(0, int(self.width() / 3))

        # Make it flickable on touchscreens
        QScroller.grabGesture(self, QScroller.ScrollerGestureType.TouchGesture)

        self.setWindowTitle("Directory Viewer")

    def contextMenuEvent(self, event):
        contextMenu = QMenu(self)

        setRecordingAct = QAction("Set Active Recording", self)
        setRecordingAct.triggered.connect(self.set_active_recording)
        contextMenu.addAction(setRecordingAct)

        contextMenu.exec(event.globalPos())

    def set_active_recording(self):
        index = self.currentIndex()
        self.expand(index)

        # Change the color for the active recording using a stylesheet
        self.setStyleSheet("QTreeView::item:selected {background: red;}")

        # also color children of the active recording
        for index in self.get_children(index):  # noqa (do we want to reassign the index value while we're looping?)
            self.setStyleSheet("QTreeView::item:selected {background: pink;}")

        # Keep the parent directory expanded
        parent_index = self.model.parent(index)
        self.expand(parent_index)

        for i in range(self.model.rowCount(index)):
            self.expand(self.model.index(i, 0, index))

        # Collapse other directories
        self.collapse_siblings(index, parent_index)

    def collapse_siblings(self, index, parent_index):
        sibling = self.indexAbove(index)
        parent = self.model.filePath(parent_index)

        while sibling.isValid():
            if self.isExpanded(sibling) and self.model.filePath(sibling) != parent:
                self.collapse(sibling)
            sibling = self.indexAbove(sibling)

        sibling = self.indexBelow(index)
        while sibling.isValid():
            if self.isExpanded(sibling) and self.model.filePath(sibling) != parent:
                self.collapse(sibling)
            sibling = self.indexBelow(sibling)

    def get_children(self, parent_index):
        children = []
        for i in range(self.model.rowCount(parent_index)):
            child_index = self.model.index(i, 0, parent_index)
            children.append(child_index)
        return children


if __name__ == "__main__":
    app = QApplication([])
    viewer = DirectoryViewer()
    viewer.show()
    sys.exit(app.exec())



import logging

import toml

from freemocap.data_layer.recording_models.recording_info_model import RecordingInfoModel
from freemocap.system.paths_and_filenames.path_getters import get_most_recent_recording_toml_path

logger = logging.getLogger(__name__)


def update_most_recent_recording_toml(recording_info_model: RecordingInfoModel):
    """Save the most recent recording path to a toml file"""

    output_file_path = get_most_recent_recording_toml_path()

    logger.info(
        f"Saving most recent recording path {str(recording_info_model.path)} to toml file: {str(output_file_path)}"
    )
    toml_dict = {}
    toml_dict["most_recent_recording_path"] = str(recording_info_model.path)
    toml_dict["recording_status"] = recording_info_model.status_check

    with open(str(output_file_path), "w") as toml_file:
        toml.dump(toml_dict, toml_file)



import logging
from pathlib import Path
from typing import Optional, Union

from freemocap.core_processes.export_data.blender_stuff.export_to_blender.export_to_blender import export_to_blender
from freemocap.core_processes.export_data.blender_stuff.get_best_guess_of_blender_path import (
    get_best_guess_of_blender_path,
)
from freemocap.core_processes.process_motion_capture_videos.process_recording_folder import process_recording_folder
from freemocap.data_layer.generate_jupyter_notebook.generate_jupyter_notebook import generate_jupyter_notebook
from freemocap.data_layer.recording_models.post_processing_parameter_models import ProcessingParameterModel
from freemocap.data_layer.recording_models.recording_info_model import RecordingInfoModel
from freemocap.system.paths_and_filenames.file_and_folder_names import RECORDING_PARAMETERS_JSON_FILE_NAME
from freemocap.system.paths_and_filenames.path_getters import get_blender_file_path
from freemocap.utilities.download_sample_data import get_sample_data_path
from freemocap.utilities.get_video_paths import get_video_paths
from freemocap.utilities.save_dictionary_to_json import save_dictionary_to_json

logger = logging.getLogger(__name__)


def process_recording_headless(
        recording_path: Union[str, Path],
        path_to_camera_calibration_toml: Optional[Union[str, Path]] = None,
        path_to_blender_executable: Optional[Union[str, Path]] = None,
        recording_processing_parameter_model: Optional[ProcessingParameterModel] = None,
        recording_info_model: Optional[RecordingInfoModel] = None,
        run_blender: bool = True,
        make_jupyter_notebook: bool = True,
        use_tqdm: bool = True,
):
    if path_to_blender_executable is None:
        path_to_blender_executable = get_best_guess_of_blender_path()
    if recording_processing_parameter_model is None:
        recording_processing_parameter_model = ProcessingParameterModel()
    rec = recording_processing_parameter_model

    if recording_info_model is None:
        rec.recording_info_model = RecordingInfoModel(recording_folder_path=Path(recording_path))
    else:
        rec.recording_info_model = recording_info_model

    logger.info(
        f"Processing recording:\n"
        f"Recording path: {recording_path}\n"
        f"Camera calibration toml path: {path_to_camera_calibration_toml}\n"
        f"Blender executable path: {path_to_blender_executable}\n"
        f"Recording processing parameter model: {rec.model_dump()}"
    )

    rec.recording_info_model = RecordingInfoModel(recording_folder_path=Path(recording_path))

    if path_to_camera_calibration_toml:
        rec.recording_info_model.calibration_toml_path = Path(path_to_camera_calibration_toml)

    if rec.recording_info_model.calibration_toml_path is None:
        number_of_videos = len(get_video_paths(rec.recording_info_model.synchronized_videos_folder_path))
        if number_of_videos > 1:
            raise ValueError(
                f"There are {number_of_videos} videos. Must provide a calibration toml file for multicamera recordings."
            )

    recording_info_dict = rec.model_dump(exclude={"recording_info_model"})

    Path(rec.recording_info_model.output_data_folder_path).mkdir(parents=True, exist_ok=True)

    save_dictionary_to_json(
        save_path=rec.recording_info_model.output_data_folder_path,
        file_name=RECORDING_PARAMETERS_JSON_FILE_NAME,
        dictionary=recording_info_dict,
    )

    logger.info("Starting core processing pipeline...")
    process_recording_folder(recording_processing_parameter_model=rec, use_tqdm=use_tqdm)

    if make_jupyter_notebook:
        logger.info("Generating jupyter notebook...")
        generate_jupyter_notebook(path_to_recording=recording_path)

    if path_to_blender_executable and run_blender:
        blender_file_path = get_blender_file_path(recording_folder_path=recording_path)
        logger.info(f"Exporting to {blender_file_path}")
        export_to_blender(
            recording_folder_path=recording_path,
            blender_file_path=blender_file_path,
            blender_exe_path=Path(path_to_blender_executable),
        )
    elif not run_blender:
        logger.info("Skipping blender export")
    else:
        logger.warning("No blender executable provided. Blender file will not be exported.")


def find_calibration_toml_path(recording_path: Union[str, Path]) -> Path:
    for file in Path(recording_path).glob("*calibration.toml"):
        return Path(file)


if __name__ == "__main__":
    recording_path = Path("PATH/TO/RECORDING/FOLDER")
    blender_path = Path("PATH/TO/BLENDER/EXECUTABLE")

    process_recording_headless(recording_path=get_sample_data_path())



import numpy as np


def rotate_by_90_degrees_around_x_axis(raw_skel3d_frame_marker_xyz: np.ndarray) -> np.ndarray:
    if len(raw_skel3d_frame_marker_xyz.shape) != 3:
        raise ValueError("raw_skel3d_frame_marker_xyz must have shape (N, M, 3)")

    swapped_skel3d_frame_marker_xyz = np.zeros(raw_skel3d_frame_marker_xyz.shape)

    swapped_skel3d_frame_marker_xyz[:, :, 0] = raw_skel3d_frame_marker_xyz[:, :, 0]
    swapped_skel3d_frame_marker_xyz[:, :, 1] = raw_skel3d_frame_marker_xyz[:, :, 2]
    swapped_skel3d_frame_marker_xyz[:, :, 2] = -raw_skel3d_frame_marker_xyz[:, :, 1]

    return swapped_skel3d_frame_marker_xyz



import threading

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QVBoxLayout, QDialog, QCheckBox, QPushButton, QLabel, QHBoxLayout, QFrame

from freemocap.gui.qt.utilities.save_and_load_gui_state import GuiState, save_gui_state
from freemocap.system.paths_and_filenames.file_and_folder_names import (
    DOCUMENTATION_MULTI_CAMERA_URL,
    DOCUMENTATION_SINGLE_CAMERA_URL,
    SPARKLES_EMOJI_STRING,
)
from freemocap.system.paths_and_filenames.path_getters import get_gui_state_json_path


class WelcomeScreenDialog(QDialog):
    def __init__(self, gui_state: GuiState, kill_thread_event: threading.Event, parent=None) -> None:
        super().__init__(parent=parent)
        self.gui_state = gui_state
        self.kill_thread_event = kill_thread_event

        self.setMinimumSize(600, 300)

        self.setWindowTitle("Welcome to Freemocap!")

        self._layout = QVBoxLayout()
        self.setLayout(self._layout)

        welcome_message = SPARKLES_EMOJI_STRING + "Welcome to Freemocap!" + SPARKLES_EMOJI_STRING

        welcome_text_label = QLabel(welcome_message)
        welcome_text_label.setWordWrap(True)
        welcome_text_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self._layout.addWidget(welcome_text_label, 1)

        self.frame = QFrame()
        self.frame.setFrameShape(QFrame.Shape.HLine)
        self.frame.setFrameShadow(QFrame.Shadow.Sunken)
        self._layout.addWidget(self.frame)

        single_camera_doc_text = "Starting with a single camera recording is a great way to learn to use FreeMoCap.\nClick the link below for a tutorial on getting 2d data from a single camera, without any calibrating."
        single_camera_docs_label = QLabel(single_camera_doc_text)
        single_camera_docs_label.setWordWrap(True)
        self._layout.addWidget(single_camera_docs_label, 1)

        single_camera_recording_doc_link_string = f'&#10132; <a href="{DOCUMENTATION_SINGLE_CAMERA_URL}" style="color: #333333;"> Single camera recording tutorial</a>'
        single_camera_doc_link = QLabel(single_camera_recording_doc_link_string)
        single_camera_doc_link.setOpenExternalLinks(True)
        self._layout.addWidget(single_camera_doc_link)

        multi_camera_doc_text = "To get 3d data from FreeMoCap, you need to record with multiple cameras and calibrate them with a charuco board.\nClick the link below for a tutorial on calibrating multiple cameras."
        multi_camera_docs_label = QLabel(multi_camera_doc_text)
        multi_camera_docs_label.setWordWrap(True)
        self._layout.addWidget(multi_camera_docs_label, 1)

        multi_camera_recording_doc_link_string = f'&#10132; <a href="{DOCUMENTATION_MULTI_CAMERA_URL}" style="color: #333333;"> Multi camera recording tutorial</a>'
        multi_camera_doc_link = QLabel(multi_camera_recording_doc_link_string)
        multi_camera_doc_link.setOpenExternalLinks(True)
        self._layout.addWidget(multi_camera_doc_link)

        sample_data_text = "You can also download sample data from the File Menu to try processing a session and see what the output looks like."
        sample_data_label = QLabel(sample_data_text)
        sample_data_label.setWordWrap(True)
        self._layout.addWidget(sample_data_label, 1)

        button_box = QHBoxLayout()

        self._dont_show_again_checkbox = QCheckBox("Don't show this again")
        self._dont_show_again_checkbox.setChecked(False)
        self._dont_show_again_checkbox.stateChanged.connect(self._dont_show_again_checkbox_changed)
        button_box.addWidget(self._dont_show_again_checkbox)

        done_button = QPushButton("Done")
        done_button.clicked.connect(self.accept)
        button_box.addWidget(done_button)

        self._layout.addLayout(button_box)

    def _dont_show_again_checkbox_changed(self) -> None:
        self.gui_state.show_welcome_screen = not self._dont_show_again_checkbox.isChecked()
        save_gui_state(gui_state=self.gui_state, file_pathstring=get_gui_state_json_path())



from fastapi import APIRouter

home_router = APIRouter()


@home_router.get("/")
async def hello():
    response_string = "hello :D"
    print(f"response string: {response_string}")
    return response_string



import logging
from dataclasses import dataclass
from pathlib import Path

from freemocap.system.paths_and_filenames.file_and_folder_names import PATH_TO_FREEMOCAP_LOGO_SVG

logger = logging.getLogger(__name__)

SKELLY_LOGO_BASE_SVG_FILENAME = "freemocap-logo-black-border.svg"
SKELLY_SWEAT_PNG = PATH_TO_FREEMOCAP_LOGO_SVG.replace(
    SKELLY_LOGO_BASE_SVG_FILENAME, "skelly-sweat.png")
SKELLY_HEART_EYES_PNG = PATH_TO_FREEMOCAP_LOGO_SVG.replace(
    SKELLY_LOGO_BASE_SVG_FILENAME, "skelly-heart-eyes.png")
SKELLY_THIS_WAY_UP_PNG = PATH_TO_FREEMOCAP_LOGO_SVG.replace(
    SKELLY_LOGO_BASE_SVG_FILENAME, "skelly-this-way-up.png")

CHARUCO_AS_GROUND_PLANE_PNG = Path(PATH_TO_FREEMOCAP_LOGO_SVG).parent.parent / "charuco/charuco_as_groundplane.png"

if not Path(SKELLY_SWEAT_PNG).exists():
    logger.warning(f"Could not find {SKELLY_SWEAT_PNG}")
if not Path(SKELLY_HEART_EYES_PNG).exists():
    logger.warning(f"Could not find {SKELLY_HEART_EYES_PNG}")
if not Path(SKELLY_THIS_WAY_UP_PNG).exists():
    logger.warning(f"Could not find {SKELLY_THIS_WAY_UP_PNG}")
if not Path(CHARUCO_AS_GROUND_PLANE_PNG).exists():
    logger.warning(f"Could not find {CHARUCO_AS_GROUND_PLANE_PNG}")


@dataclass
class ReleaseNoteContent:
    """Data class to hold content for a release note tab."""

    tab_title: str
    content_title: str
    content_html: str
    content_subtitle: str | None = None
    image_path: str | None = None
    logo_path: str | None = None
    tab_order: int = 0  # Lower numbers appear first
    latest: bool = False


def get_v170_release_notes() -> ReleaseNoteContent:
    """Return the release notes for v1.7.0."""

    return ReleaseNoteContent(
        latest=True,
        logo_path=SKELLY_THIS_WAY_UP_PNG,  # Using the "this way up" logo since it's about orientation
        image_path=str(CHARUCO_AS_GROUND_PLANE_PNG),
        tab_order=0,
        tab_title="v1.7.0 Release Notes",
        content_title="FreeMoCap v1.7.0 - Ground Plane Calibration!",
        content_subtitle="✨ New Feature: Ground Plane Calibration (and new board definitions)",
        content_html="""
<style>
    p {{ margin: 5px 0; }}
    ul, ol {{ margin: 10px 0 10px 20px; }}
    li {{ margin: 3px 0; }}
    a {{ color: #4fc3f7; text-decoration: none; }}
    b {{ font-weight: bold; }}
    em {{ font-style: italic; }}
    .emphasis {{ font-weight: bold; }}
    .note {{ margin: 10px 0; padding: 8px; background-color: #884; }}
</style>


We have (finally!) added a new feature to allow you to define the ground plane of your capture volume during calibration
based on the initial position of a ChArUco board.
</p>

<p>
    <b>How to use it:</b>
</p>
<ol>
    <li>Check the <span class="emphasis">"Use board position as origin"</span> option before calibration</li>
    <li>Start your calibration recording with the board flat on the ground (visible to as many cameras as possible) for a few seconds</li>
    <li>Continue calibration as normal - that's it!</li>
</ol>
<p>
    This new feature automatically aligns your 3D Capture Volume to the detected board position, ensuring:
</p>
<ul>
    <li>The origin is at the charco corner #0</li>
    <li>The X+ axis points along the board's SHORT axis</li>
    <li>The Y+ axis points along the board's LONG axis</li>
    <li>The Z+ axis points UPWARDS normal to the board (XxY)</li>
</ul>

<p>
    Previously, the reference frame resulting from our customized  <a href="https://github.com/lambdaloop/aniposelib">anipose</a>
    based calibration method was defined based on the 6DoF position of the first camera,
    and we defined the floor post-hoc in each recording based on detected foot location.
</p>

<p>
    This process could fail if the feet weren't detected well, resulting the data returning a random-ish orientation.
    (NOTE:
    You can correct this kind of error by adjusting the position of the recording parent Empty in the generated
    `[recording_name].blend` Blender scene).
</p>

<p>
    The new calibration method sets the capture volume orientation at the level of <em>CALIBRATION</em> file, which keeps the reference frame
consistent across recordings.
</p>

<p class="note">
    <b>Note:</b> If ground plane calibration fails (board not visible or moving), the system automatically falls back
    to the standard calibration method.
</p>

<p>
For detailed instructions and troubleshooting, check out the  <a
        href="https://docs.freemocap.org/documentation/groundplane-calibration.html"> <em>Ground Plane Calibration Guide</em></a>.
</p>

<hr/>

<p>
    <em>🆕 New 5x3 ChArUco board definition </em>
</p>

<p>
In addition, our calibration process now supports a new 5x3 ChArUco board (5 rows, 3 columns)!
</p>

<p>
This definition will print larger Aruco patterns than the original 7x5 pattern for a given paper size, which will make it
easier to calibrate larger spaces without needing to construct or print
a poster-sized board.
</p>

<p>
For details or to download high resolution images board images, see  <a href="https://docs.freemocap.org/documentation/multi-camera-calibration.html#charuco-board-types"><em>Preparing the Charuco Board</em></a>.
</p>

<hr/>

<p>
For additional details about what's new in v1.7.0 - See <a
        href="https://github.com/freemocap/freemocap/releases/tag/v1.7.0">the official release page</a>
</p>

<p>
(You can always access these release notes from the <b>Help</b> menu.)
</p>
        """,
    )


def get_v160_release_notes() -> ReleaseNoteContent:
    """Return the release notes for v1.6.0."""

    return ReleaseNoteContent(
        latest=False,  # No longer the latest
        tab_title="v1.6.0 Data Scaling",
        content_title="FreeMoCap v1.6.0 - Data Scaling improvement!",
        content_html="""
            <html>
            <style>
            a {{ color: #ccc; text-decoration: none; font-weight: 500; }}
            .emphasis {{ color: #aaa; font-weight: bold; }}
            </style>
            <body>
            <p>So, remember last time when we said we were going to add a <a href="https://github.com/freemocap/freemocap/pull/676"> set of comprehensive quality assurance diagnostics </a> to ensure monotonically increasing data quality?
            <br/><br/>

            Well, we did that and immediately discovered a bug in our calibration/reconstruction pipeline that has apparently been causing a ~10-15% scaling offset to our data (i.e. so measured limb segment lengths would be ~85-115% of their real-world lengths)
            <br/><br/>
           Good news is that we fixed it, so our data should be more accurately scaled to real-world units! We call that progress 😌
            <br/><br/>
            If your application requires empirical accuracy, we recommend reprocessing any critical data with the latest version of FreeMoCap to ensure the highest quality results.
            <br/>
            <br>
            See  <a href="https://github.com/freemocap/freemocap/pull/681">Issue #681</a> for a full run down on the code sleuthing and thread pulling that led to this fix.
            <br>
            --
            <br>
            For additional details about what's new in v1.6.3 - See <a href="https://github.com/freemocap/freemocap/releases/tag/v1.6.3">the official release page </a>
            <br/><br/>
            (You can always access these release notes from the <b>Help</b> menu.)

            </body>
            </html>
        """,
        logo_path=SKELLY_HEART_EYES_PNG,
        tab_order=1,  # Now second
    )


def get_v154_release_notes() -> ReleaseNoteContent:
    """Return the release notes for v1.5.4 (Butterworth fix)."""

    return ReleaseNoteContent(
        tab_title="v1.5.4 Butterworth Filter Fix",
        content_title="Butterworth Filter Fix in v1.5.4",
        content_html="""
            <html>
            <style>
            a {{ color: #ccc; text-decoration: none; font-weight: 500; }}
            .emphasis {{ color: #aaa; font-weight: bold; }}
            </style>
            <body>
            <p style="font-size: 18px; font-weight: bold; color: #f5a; margin-bottom: 15px;">
                Whoops!!
            </p>
            <p style="font-size: 16px; font-weight: semibold; color: #fc8;   margin-bottom: 15px;">
                Possible data Quality Regression in versions 1.4.7-1.5.3
            </p>

            <p>We identified and fixed a bug in <b>v1.4.7-v1.5.3 (10 Oct 2024 - 10 Mar 2025)</b> that caused the pipeline to Butterworth filtering during processing (<a href="https://github.com/freemocap/freemocap/pull/675">Bugfix PR</a>).
            Recordings from these versions may have increased noise/jitter/shakiness in the final keypoint trajectories.
            <br/><br/>
            Based on your application, the difference may or may not be noticeable. It is most likely to affect users who's applications focus on fine grained trajectories of the hands and limbs (especially for scientific analysis)
            <br/><br/>
            <b>We recommend reprocessing any critical data collected during this period with the latest version of FreeMoCap to ensure the highest quality results.</b>
            <br/><br/>
             You may also filter the data in Blender (see <a href="https://www.youtube.com/watch?v=33OhM5xFUlg">this tutorial Flux Renders</a>)
            <br/>
            --
            <br>
            In preparation for the release of FreeMoCap v2.0 (optimistically Summer 2025), we are implementing a <a href="https://github.com/freemocap/freemocap/pull/676"> set of comprehensive quality assurance diagnostics </a>to ensure that the quality of our output is strictly monotonic across future versions.

            <p style="font-size: 13px; margin-top: 20px; color: #7f8c8d;">
            Thanks to  (<a href="https://discord.com/channels/760487252379041812/760489602917466133/1346487740568440983">@larap for reporting</a>), and to the rest of the freemocap community for their help in developing this project.
            </p>
            </body>
            </html>
        """,
        logo_path=SKELLY_SWEAT_PNG,
        tab_order=2,  # Now third
    )


def get_older_versions_content() -> ReleaseNoteContent:
    """Return the content for older versions tab."""
    return ReleaseNoteContent(
        tab_title="Older Versions",
        content_title="Previous Versions",
        content_html="""
        <p>For information about older versions, please visit our
        <a href='https://github.com/freemocap/freemocap/releases'>GitHub releases page</a>.</p>
        """,
        logo_path=None,
        tab_order=3,  # Now fourth
    )


def get_all_release_notes() -> list[ReleaseNoteContent]:
    """Return all release notes in the correct order."""
    release_notes = [
        get_v170_release_notes(),
        get_v160_release_notes(),
        get_v154_release_notes(),
        get_older_versions_content(),
    ]

    # Sort by tab_order
    return sorted(release_notes, key=lambda x: x.tab_order)



import multiprocessing
from pathlib import Path

from freemocap.data_layer.recording_models.post_processing_parameter_models import ProcessingParameterModel, AniposeTriangulate3DParametersModel, PostProcessingParametersModel
from freemocap.data_layer.recording_models.recording_info_model import RecordingInfoModel
from freemocap.core_processes.process_motion_capture_videos.process_recording_folder import process_recording_folder
from skellytracker.trackers.yolo_tracker.yolo_model_info import YOLOTrackingParams, YOLOModelInfo

def process_videos_YOLO(recording_folder: Path, num_processes: int = multiprocessing.cpu_count() - 1):
    """
    Process a recording folder from the synchronized videos
    """
    recording_model = RecordingInfoModel(
        recording_folder_path=recording_folder,
        active_tracker="yolo"
    )

    yolo_processing_parameters = ProcessingParameterModel(
        recording_info_model=recording_model,
        tracking_parameters_model=YOLOTrackingParams(
            num_processes=num_processes,
            model_size="medium",
        ),
        anipose_triangulate_3d_parameters_model=AniposeTriangulate3DParametersModel(),
        post_processing_parameters_model=PostProcessingParametersModel(),
        tracking_model_info=YOLOModelInfo(),
    )

    process_recording_folder(
        recording_processing_parameter_model=yolo_processing_parameters,
        kill_event=None,
        logging_queue=None,
    )

def process_raw_data_YOLO(recording_folder: Path):
    """
    Process a recording folder where raw, 2d data is already available
    """
    recording_model = RecordingInfoModel(
        recording_folder_path=recording_folder,
        active_tracker="yolo"
    )

    yolo_processing_parameters = ProcessingParameterModel(
        recording_info_model=recording_model,
        tracking_parameters_model=YOLOTrackingParams(
            run_image_tracking=False,
        ),
        anipose_triangulate_3d_parameters_model=AniposeTriangulate3DParametersModel(),
        post_processing_parameters_model=PostProcessingParametersModel(),
        tracking_model_info=YOLOModelInfo(),
    )

    process_recording_folder(
        recording_processing_parameter_model=yolo_processing_parameters,
        kill_event=None,
        logging_queue=None,
    )

if __name__ == "__main__":
    recording_folder = Path("/Users/philipqueen/freemocap_data/recording_sessions/freemocap_test_data/")
    process_videos_YOLO(recording_folder=recording_folder)



import logging
from pathlib import Path
from typing import Union

from PySide6 import QtCore
from PySide6.QtCore import Slot, QFileSystemWatcher

logger = logging.getLogger(__name__)


class CSSFileWatcher(QFileSystemWatcher):
    def __init__(self, path_to_css_file: Union[str, Path], parent=None):
        super().__init__(parent)
        self.addPath(str(path_to_css_file))
        self.parent = parent
        self.fileChanged.connect(self.file_changed)

    @Slot(str)
    def file_changed(self, path: str):
        logger.info(f"CSS file changed: {path} - updating MainWindow stylesheet")

        try:
            with open(path, "r") as css_file:
                logger.info(f"Reading CSS file from: {path}")
                css_string = css_file.read()
                logger.info(f"Setting stylesheet for {self.parent}")
                self.parent.setStyleSheet(css_string)
        except Exception as e:
            logger.error(f"Error setting stylesheet: {e}")


if __name__ == "__main__":
    app = QtCore.QCoreApplication([])
    qt_file_system_watcher = CSSFileWatcher(path_to_css_file=Path(__file__).parent / "qt_style_sheet.css")
    print(f"watching directories - {qt_file_system_watcher.directories()})")
    print(f"watching files - {qt_file_system_watcher.files()})")
    app.exec()
    print("done!")



import numpy as np
import pytest

from freemocap.utilities.geometry.project_3d_data_to_z_plane import project_3d_data_to_z_plane
from freemocap.utilities.geometry.rotate_by_90_degrees_around_x_axis import rotate_by_90_degrees_around_x_axis


def test_rotate_by_90_degrees_around_x_axis():
    # test single point
    raw_skel3d_frame_marker_xyz = np.array([[[1, 2, 3]]])
    expected_result = np.array([[[1, 3, -2]]])
    assert np.allclose(
        rotate_by_90_degrees_around_x_axis(raw_skel3d_frame_marker_xyz), expected_result
    ), "Did not correctly rotate the single input point."

    # test multiple points
    raw_skel3d_frame_marker_xyz = np.array([[[1, 2, 3], [4, 5, 6], [7, 8, 9]]])
    expected_result = np.array([[[1, 3, -2], [4, 6, -5], [7, 9, -8]]])
    assert np.allclose(
        rotate_by_90_degrees_around_x_axis(raw_skel3d_frame_marker_xyz), expected_result
    ), "Did not correctly rotate the multiple input points."

    # test empty input
    raw_skel3d_frame_marker_xyz = np.array([])
    with pytest.raises(ValueError) as exc_info:
        rotate_by_90_degrees_around_x_axis(raw_skel3d_frame_marker_xyz)
    assert (
            str(exc_info.value) == "raw_skel3d_frame_marker_xyz must have shape (N, M, 3)"
    ), "Did not correctly handle an empty input array."

    # test zero input
    raw_skel3d_frame_marker_xyz = np.array([[[0, 0, 0], [0, 0, 0], [0, 0, 0]]])
    expected_result = np.array([[[0, 0, 0], [0, 0, 0], [0, 0, 0]]])
    assert np.allclose(
        rotate_by_90_degrees_around_x_axis(raw_skel3d_frame_marker_xyz), expected_result
    ), "Did not correctly handle a zero input array."


def test_project_3d_data_to_z_plane():
    # test simple case
    input_data = np.array([[[1, 2, 3], [4, 5, 6]], [[7, 8, 9], [10, 11, 12]]])
    expected_output = np.array([[[1, 2, 0], [4, 5, 0]], [[7, 8, 0], [10, 11, 0]]])
    assert np.array_equal(
        project_3d_data_to_z_plane(input_data), expected_output
    ), f"Test failed for input_data: {input_data}"

    # test more complex case
    input_data = np.array(
        [
            [[7.80, 5.67, 1.72], [9.71, 9.42, 2.57]],
            [[4.85, 4.38, 3.42], [3.26, 1.13, 1.27]],
            [[3.70, 1.02, 8.64], [4.85, 5.10, 8.96]],
            [[6.57, 2.48, 2.52], [5.08, 1.76, 9.34]],
        ]
    )
    input_data_copy = np.array(
        [
            [[7.80, 5.67, 1.72], [9.71, 9.42, 2.57]],
            [[4.85, 4.38, 3.42], [3.26, 1.13, 1.27]],
            [[3.70, 1.02, 8.64], [4.85, 5.10, 8.96]],
            [[6.57, 2.48, 2.52], [5.08, 1.76, 9.34]],
        ]
    )
    expected_output = np.array(
        [
            [[7.80, 5.67, 0], [9.71, 9.42, 0]],
            [[4.85, 4.38, 0], [3.26, 1.13, 0]],
            [[3.70, 1.02, 0], [4.85, 5.10, 0]],
            [[6.57, 2.48, 0], [5.08, 1.76, 0]],
        ]
    )
    assert np.array_equal(
        project_3d_data_to_z_plane(input_data), expected_output
    ), f"Test failed for input_data: {input_data}"

    # test input data was not changed
    assert np.array_equal(
        input_data, input_data_copy
    ), "Input data was modified by function, function should not change the input data."



import multiprocessing
from logging.handlers import QueueHandler


class DirectQueueHandler(QueueHandler):
    def __init__(self, queue: multiprocessing.Queue):
        super().__init__(queue)

    def enqueue(self, record):
        self.queue.put(record)



import logging
from pathlib import Path
from typing import Union

from freemocap.core_processes.export_data.blender_stuff.export_to_blender.methods.ajc_addon.run_ajc_addon_main import (
    run_ajc_blender_addon_subprocess,
)
from freemocap.core_processes.export_data.blender_stuff.get_best_guess_of_blender_path import (
    get_best_guess_of_blender_path,
)

logger = logging.getLogger(__name__)


def export_to_blender(
        recording_folder_path: Union[str, Path],
        blender_file_path: Union[str, Path],
        blender_exe_path: Union[str, Path],
        method: str = "ajc27_blender_addon",
):
    logger.info(
        f"Exporting session data to a Blender scene at: {str(blender_file_path)} using Blender executable at {str(blender_exe_path)}"
    )

    if method == "ajc27_blender_addon":
        run_ajc_blender_addon_subprocess(
            recording_folder_path=recording_folder_path,
            blender_file_path=blender_file_path,
            blender_exe_path=blender_exe_path,
        )

    logger.info("Done with Blender Export :D")


if __name__ == "__main__":
    recording_path = r"D:\Dropbox\FreeMoCapProject\FreeMocap_Data\old\sesh_2022-09-19_16_16_50_in_class_jsm"
    export_to_blender(recording_path, recording_path, get_best_guess_of_blender_path())



import logging
import re
from pathlib import Path

from PySide6.QtWidgets import (
    QVBoxLayout,
    QPushButton,
    QWidget,
    QCheckBox,
    QLabel,
    QFileDialog,
)

from freemocap.gui.qt.utilities.save_and_load_gui_state import GuiState, save_gui_state
from freemocap.system.paths_and_filenames.path_getters import get_gui_state_json_path

logger = logging.getLogger(__name__)
BLENDER_EXECUTABLE_PATH_MISSING_STRING = "BLENDER EXECUTABLE NOT FOUND"


class VisualizationControlPanel(QWidget):
    def __init__(self, gui_state: GuiState, parent=None):
        super().__init__(parent=parent)

        self._gui_state = gui_state
        self._blender_executable_path = str(self._gui_state.blender_path)
        self._layout = QVBoxLayout()
        self.setLayout(self._layout)

        self._generate_jupyter_notebook = QPushButton("Generate Jupyter Notebook")
        self._layout.addWidget(self._generate_jupyter_notebook)

        self._export_to_blender_button = QPushButton("Export to Blender")
        self.export_to_blender_button.setObjectName("export_to_blender_button")
        self._layout.addWidget(self._export_to_blender_button)

        self._open_in_blender_automatically_checkbox = QCheckBox("Open in Blender automatically")
        self._open_in_blender_automatically_checkbox.setChecked(True)

        self._layout.addWidget(QLabel("Blender Executable Path:"))

        self._blender_executable_label = QLabel(self._blender_executable_path)
        self._layout.addWidget(self._blender_executable_label)

        self._set_blender_executable_path_button = QPushButton("Choose your Blender Executable")
        self._set_blender_executable_path_button.clicked.connect(self._handle_blender_executable_button_clicked)
        self._layout.addWidget(self._set_blender_executable_path_button)
        self._layout.addStretch()

    @property
    def export_to_blender_button(self):
        return self._export_to_blender_button

    @property
    def generate_jupyter_notebook_button(self):
        return self._generate_jupyter_notebook

    @property
    def open_in_blender_automatically_box_is_checked(self):
        return self._open_in_blender_automatically_checkbox.isChecked()

    @property
    def blender_executable_path(self) -> str:
        """
        Returns the path to the Blender executable

        Returns:
            str: Path to the Blender executable
        """
        return self._blender_executable_path

    def _handle_blender_executable_button_clicked(self):
        # from this tutorial - https://www.youtube.com/watch?v=gg5TepTc2Jg&t=649s
        path_selection = QFileDialog.getOpenFileName(
            self,
            "Locate your Blender Executable",
            str(Path().home()),
            "*",
        )

        if "blender" in path_selection[0]:
            self._blender_executable_path = path_selection[0]
        elif re.match(r".*Blender.*\.app$", path_selection[0]):
            self._blender_executable_path = path_selection[0] + "/Contents/MacOS/Blender"  # executable is buried on mac
        else:
            self._blender_executable_path = BLENDER_EXECUTABLE_PATH_MISSING_STRING

        logger.info(f"User selected Blender Executable path:{self._blender_executable_path}")
        self._blender_executable_label.setText(self._blender_executable_path)

        if (
                self._blender_executable_path != BLENDER_EXECUTABLE_PATH_MISSING_STRING
        ):  # don't persist missing paths across sessions
            self._gui_state.blender_path = self._blender_executable_path
            save_gui_state(gui_state=self._gui_state, file_pathstring=get_gui_state_json_path())



import logging
import signal
import sys
from importlib.metadata import distributions
from pathlib import Path

from PySide6.QtCore import QTimer
from PySide6.QtWidgets import QApplication

from freemocap.gui.qt.main_window.freemocap_main_window import MainWindow, EXIT_CODE_REBOOT

logging.getLogger("matplotlib").setLevel(logging.WARNING)

from freemocap.gui.qt.utilities.get_qt_app import get_qt_app
from freemocap.system.paths_and_filenames.path_getters import get_freemocap_data_folder_path
from freemocap.system.user_data.pipedream_pings import PipedreamPings

repo = Path(__file__).parent.parent.parent.parent
sys.path.insert(0, str(repo))

logger = logging.getLogger(__name__)


def sigint_handler(*args):
    """Handler for the SIGINT signal."""
    QApplication.quit()


def qt_gui_main():
    logger.info("Starting main...")
    signal.signal(signal.SIGINT, sigint_handler)
    app = get_qt_app()
    timer = QTimer()
    timer.start(500)

    pipedream_pings = PipedreamPings()

    while True:
        freemocap_main_window = MainWindow(
            freemocap_data_folder_path=get_freemocap_data_folder_path(), pipedream_pings=pipedream_pings
        )
        logger.info("Showing main window - Ready to start!")

        freemocap_main_window.show()

        handle_pop_ups(freemocap_main_window)

        timer.timeout.connect(freemocap_main_window.update)
        error_code = app.exec()
        logger.info(f"`main` exited with error code: {error_code}")
        freemocap_main_window.close()

        if not error_code == EXIT_CODE_REBOOT:
            print("Thank you for using freemocap \U0001f480 \U00002764 \U00002728")
            break

        logger.info("`main` exited with the 'reboot' code, so let's reboot!")

    sys.exit()


def handle_pop_ups(freemocap_main_window: MainWindow):
    if freemocap_main_window._gui_state.show_welcome_screen:
        freemocap_main_window.open_welcome_screen_dialog()
    if not freemocap_main_window._gui_state.shown_latest_release_notes:
        freemocap_main_window.open_release_notes_popup()

    installed_packages = {dist.metadata["Name"] for dist in distributions()}
    if "opencv-python" in installed_packages and "opencv-contrib-python" in installed_packages:
        freemocap_main_window.open_opencv_conflict_dialog()


if __name__ == "__main__":
    qt_gui_main()



import inspect
import logging
import subprocess
from pathlib import Path
from typing import Union, List

from ajc27_freemocap_blender_addon.main import ajc27_run_as_main_function

from freemocap.core_processes.export_data.blender_stuff.export_to_blender.methods.ajc_addon.get_numpy_path import (
    get_numpy_path,
)
from freemocap.core_processes.export_data.blender_stuff.export_to_blender.methods.ajc_addon.install.install_ajc_addon import (
    install_ajc_addon,
)
from freemocap.core_processes.export_data.blender_stuff.export_to_blender.methods.ajc_addon.run_simple import run_simple
from freemocap.system.paths_and_filenames.file_and_folder_names import (
    OLD_TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME,
    OLD_SEGMENT_CENTER_OF_MASS_NPY_FILE_NAME,
    OLD_REPROJECTION_ERROR_NPY_FILE_NAME,
    REPROJECTION_ERROR_NPY_FILE_NAME,
    SEGMENT_CENTER_OF_MASS_NPY_FILE_NAME,
    TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME,
)

logger = logging.getLogger(__name__)


def run_subprocess(command_list: List[str], append_to_python_path: List[str] = None):
    logger.debug(f"Running subprocess with command list: {command_list}")
    # modified_env = os.environ.copy()
    # if append_to_python_path is not None:
    #     logger.debug(f"Appending to PYTHONPATH: {append_to_python_path}")
    #     for addon_root_directory in append_to_python_path:
    #         if not Path(addon_root_directory).exists():
    #             raise FileNotFoundError(f"Could not find addon root directory at {addon_root_directory}")
    #         modified_env["PYTHONPATH"] += f";{addon_root_directory}"
    process = subprocess.Popen(command_list, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    # env=modified_env)
    return process


def run_ajc_blender_addon_subprocess(
        recording_folder_path: Union[str, Path],
        blender_file_path: Union[str, Path],
        blender_exe_path: Union[str, Path],
):
    try:
        # TODO: pass active tracker into here
        ajc_blender_addon_validator(recording_folder_path=recording_folder_path)
    except FileNotFoundError as e:
        logger.error("Missing required files to run AJC addon, did something go wrong during processing?")
        raise e

    ajc_addon_main_file_path = inspect.getfile(ajc27_run_as_main_function)
    logger.debug(f"Running ajc27_freemocap_blender_addon as a subprocess using script at : {ajc_addon_main_file_path}")

    addon_root_directory = str(Path(ajc_addon_main_file_path).parent.parent)

    # path_to_blenders_numpy = get_blenders_numpy(blender_exe_path=blender_exe_path)
    # blender_numpy_root = str(Path(path_to_blenders_numpy).parent.parent)

    install_ajc_addon(blender_exe_path=blender_exe_path, ajc_addon_main_file_path=ajc_addon_main_file_path)
    try:
        blender_exe_path = Path(blender_exe_path)
        if not blender_exe_path.exists():
            raise FileNotFoundError(f"Could not find the blender executable at {blender_exe_path}")
    except Exception as e:
        logger.error(e)
        raise e

    simple_run_script = inspect.getfile(run_simple)

    command_list = [
        str(blender_exe_path),
        "--background",
        "--python",
        simple_run_script,
        "--",
        str(recording_folder_path),
        str(blender_file_path),
    ]

    logger.info(f"Starting `blender` sub-process with this command: \n {command_list}")

    blender_process = run_subprocess(
        command_list=command_list, append_to_python_path=[addon_root_directory]
    )  # , blender_numpy_root])

    while True:
        output = blender_process.stdout.readline()
        if blender_process.poll() is not None:
            break
        if output:
            logging.debug(output.strip().decode())

    if blender_process.returncode != 0:
        logging.error(blender_process.stderr.read().decode())
    logger.debug("Done with blender add on")
    blender_process.terminate()  # manually terminate the process


def get_blenders_numpy(blender_exe_path: Union[str, Path]) -> str:
    import subprocess

    get_numpy_script_path = inspect.getfile(get_numpy_path)

    command = [
        str(blender_exe_path),
        "--background",
        "--python",
        get_numpy_script_path,
    ]
    subprocess_result = subprocess.run(command, capture_output=True, text=True)
    all_output = subprocess_result.stdout
    numpy_path = None
    for line in all_output.split("\n"):
        if "numpy" in line:
            numpy_path = line
            break
    if not Path(numpy_path).exists():
        raise FileNotFoundError(f"Could not find Blender's numpy at {numpy_path}")
    logger.info(f"Got Blender's numpy path: {numpy_path}")
    return numpy_path


def ajc_blender_addon_validator(recording_folder_path: Union[str, Path], active_tracker: str = "mediapipe"):
    """
    Check if the required files exist in the recording folder
    """
    recording_path = Path(recording_folder_path)
    output_data_path = recording_path / "output_data"

    if active_tracker[-1] != "_":
        active_tracker = active_tracker + "_"

    if not (output_data_path / (active_tracker + "body_3d_xyz.npy")).exists():
        raise FileNotFoundError(
            f"Could not find required file: {output_data_path / (active_tracker + 'body_3d_xyz.npy')}"
        )

    if not (output_data_path / (active_tracker + "right_hand_3d_xyz.npy")).exists():
        raise FileNotFoundError(
            f"Could not find required file: {output_data_path / (active_tracker + 'right_hand_3d_xyz.npy')}"
        )

    if not (output_data_path / (active_tracker + "left_hand_3d_xyz.npy")).exists():
        raise FileNotFoundError(
            f"Could not find required file: {output_data_path / (active_tracker + 'left_hand_3d_xyz.npy')}"
        )

    if not (output_data_path / (active_tracker + "face_3d_xyz.npy")).exists():
        raise FileNotFoundError(
            f"Could not find required file: {output_data_path / (active_tracker + 'face_3d_xyz.npy')}"
        )

    if not (output_data_path / "center_of_mass" / (active_tracker + TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME)).exists():
        if (
                active_tracker != "mediapipe_"
                or not (output_data_path / "center_of_mass" / OLD_TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME).exists()
        ):
            raise FileNotFoundError(
                f"Could not find required file: {output_data_path / 'center_of_mass' / (active_tracker + TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME)}"
            )

    if not (output_data_path / "center_of_mass" / (active_tracker + SEGMENT_CENTER_OF_MASS_NPY_FILE_NAME)).exists():
        if (
                active_tracker != "mediapipe_"
                or not (output_data_path / "center_of_mass" / OLD_SEGMENT_CENTER_OF_MASS_NPY_FILE_NAME).exists()
        ):
            raise FileNotFoundError(
                f"Could not find required file: {output_data_path / 'center_of_mass' / (active_tracker + SEGMENT_CENTER_OF_MASS_NPY_FILE_NAME)}"
            )

    if not (output_data_path / "raw_data" / (active_tracker + REPROJECTION_ERROR_NPY_FILE_NAME)).exists():
        if (
                active_tracker != "mediapipe_"
                or not (output_data_path / "raw_data" / OLD_REPROJECTION_ERROR_NPY_FILE_NAME).exists()
        ):
            raise FileNotFoundError(
                f"Could not find required file: {output_data_path / 'raw_data' / (active_tracker + REPROJECTION_ERROR_NPY_FILE_NAME)}"
            )


if __name__ == "__main__":
    from freemocap.core_processes.export_data.blender_stuff.get_best_guess_of_blender_path import (
        get_best_guess_of_blender_path,
    )

    recording_path_in = r"C:\Users\jonma\freemocap_data\recording_sessions\steen_pantsOn_gait"
    blender_file_path_in = str(Path(recording_path_in) / (str(Path(recording_path_in).stem) + ".blend"))
    blender_exe_path_in = get_best_guess_of_blender_path()

    run_ajc_blender_addon_subprocess(
        recording_folder_path=recording_path_in,
        blender_file_path=blender_file_path_in,
        blender_exe_path=blender_exe_path_in,
    )



import logging
import multiprocessing
import threading
from logging.handlers import QueueHandler
from queue import Queue

from PySide6 import QtCore, QtGui
from PySide6.QtCore import QThread
from PySide6.QtWidgets import QApplication, QPlainTextEdit

from freemocap.gui.qt.utilities.colors import get_next_color, rgb_color_generator
from freemocap.system.logging.configure_logging import log_view_logging_format_string
from freemocap.system.paths_and_filenames.file_and_folder_names import LOG_VIEW_PROGRESS_BAR_STRING

logger = logging.getLogger(__name__)

log_view_logging_formatter = logging.Formatter(fmt=log_view_logging_format_string, datefmt="%Y-%m-%dT%H:%M:%S")

level_colors = {
    "DEBUG": (169, 169, 169),  # dimmed gray
    "INFO": (255, 255, 255),  # white
    "WARNING": (255, 165, 0),  # orange
    "ERROR": (255, 0, 0),  # red
}

process_colors = {}
thread_colors = {}
code_path_colors = {}


class LoggingQueueListener(QThread):
    log_message_signal = QtCore.Signal(object)

    def __init__(self, logging_queue: Queue, exit_event: threading.Event, parent=None):
        super().__init__(parent)
        self._logging_queue = logging_queue
        self._exit_event = exit_event
        self._parent = parent

    def run(self):
        logger.info("Starting LoggingQueueListener thread")
        try:
            while not self._exit_event.is_set():
                if self._logging_queue.empty():
                    if self._parent._keep_logging:
                        QtCore.QMetaObject.invokeMethod(self._parent, "log_progress", QtCore.Qt.QueuedConnection)
                    continue

                record = self._logging_queue.get(block=True)

                if record is None:
                    break

                self.log_message_signal.emit(record)

        except Exception as e:
            logger.exception(e)
            self.close()

    def close(self):
        logger.info("Closing LoggingQueueListener thread")
        self._exit_event.set()


class LogViewWidget(QPlainTextEdit):
    # adapted from - https://stackoverflow.com/a/63853259/14662833
    def __init__(self, parent=None):
        super().__init__(parent)

        self.setReadOnly(True)

        self._logging_queue = multiprocessing.Queue(-1)
        self._queue_handler = QueueHandler(self._logging_queue)
        self._queue_handler.setFormatter(log_view_logging_formatter)
        self._queue_handler.setLevel(logging.INFO)
        logging.getLogger("").handlers.append(self._queue_handler)

        self._exit_event = threading.Event()
        self._logging_queue_listener = LoggingQueueListener(
            logging_queue=self._logging_queue, exit_event=self._exit_event, parent=self
        )
        self._keep_logging = False
        self._logging_queue_listener.log_message_signal.connect(self.add_log)
        self.timestamp_color_generator = rgb_color_generator((50, 255, 255), (255, 50, 255), phase_increment=0.2)
        self.log_message_color_generator = rgb_color_generator((128, 255, 100), (100, 255, 255), phase_increment=0.7)
        self.show_colors = True
        self.show_code_path_info = True
        self._logging_queue_listener.start()

    def add_log(self, record: logging.LogRecord):
        level = record.levelname.ljust(7)  # 7 characters long, right padded with spaces
        timestamp = f"[{record.asctime}.{record.msecs:04.0f}]".ljust(24)  # 24 characters long
        process_id_str = f"{record.process}".rjust(6)  # 5 characters long, left padded with spaces
        thread_id_str = f"{record.thread}".rjust(6)  # 5 characters long, left padded with spaces
        full_message = record.getMessage().split(":::::")[-1].strip()
        message_content = full_message.split("||||")[-1].strip()
        code_path_str = full_message.split("||||")[0].strip()
        package, method, line = code_path_str.split(":")

        if message_content == LOG_VIEW_PROGRESS_BAR_STRING + ".":
            self._keep_logging = True
            return
        else:
            self._keep_logging = False

        # Set color for the level
        r, g, b = level_colors.get(level.strip(), (255, 255, 255))
        color_level = f"<span style='color:rgb({r},{g},{b});'>{level}</span>"

        # Set color for the timestamp using the timestamp_color_generator
        r, g, b = next(self.timestamp_color_generator)
        color_timestamp = f"<span style='color:rgb({r},{g},{b});'>{timestamp}</span>"

        # Get color for process and thread ID
        if process_id_str not in process_colors:
            process_colors[process_id_str] = get_next_color()
        r, g, b = process_colors[process_id_str]
        process_str = f"ProcessID:{process_id_str}"

        color_process_id = f"<span style='color:rgb({r},{g},{b});'>[{process_str},</span>"

        thread_process_id_str = f"{thread_id_str}:{process_id_str}"
        if thread_process_id_str not in thread_colors:
            thread_colors[thread_process_id_str] = get_next_color()
        r, g, b = thread_colors[thread_process_id_str]
        thread_str = f"ThreadID:{thread_id_str}"
        color_thread_id = f"<span style='color:rgb({r},{g},{b});'>{thread_str}]</span>"

        color_process_id_thread_id = f"{color_process_id} {color_thread_id}"
        color_process_id_thread_id.ljust(25)

        # Choose a color for the message using the log_message_color_generator

        if package + method not in code_path_colors:
            code_path_colors[package + method] = get_next_color()

        r, g, b = code_path_colors[package + method]
        color_code_path = f"<span style='color:rgb({r},{g},{b});'>{code_path_str}</span>"

        r, g, b = (255, 255, 255)
        color_message = f"<span style='color:rgb({r},{g},{b});'>{message_content}</span>"

        # Combine colored parts
        colored_log_entry = (
            f"{color_timestamp}[{color_level}] {color_process_id_thread_id} {color_code_path} ::: {color_message}"
        )
        self.appendHtml(colored_log_entry)

    @QtCore.Slot()
    def log_progress(self, bar_length: int = 100, cycles_per_update: int = 2500) -> None:
        if not hasattr(self, "_progress_counter"):
            self._progress_counter = 0

        progress_marker = "▁▂▃▄▅▆▇▆▅▄▃▂▁"
        help_text = "Processing:"

        if self.toPlainText().split("\n")[-1].strip().split(" ")[0] != help_text:
            self.appendPlainText(help_text + " " + progress_marker * bar_length)
            return

        cycles_until_repeat = len(progress_marker) * cycles_per_update
        for i in range(len(progress_marker)):
            if self._progress_counter % cycles_until_repeat == cycles_per_update * i:
                self._replace_last_line(
                    help_text
                    + " "
                    + (progress_marker[i:] + progress_marker[:i]) * int(bar_length / len(progress_marker))
                )

        self._progress_counter += 1

    def _replace_last_line(
            self, text: str
    ) -> None:  # from https://stackoverflow.com/questions/53381975/display-terminal-output-with-tqdm-in-qplaintextedit
        cursor = self.textCursor()
        cursor.movePosition(QtGui.QTextCursor.End)
        cursor.select(QtGui.QTextCursor.BlockUnderCursor)
        cursor.removeSelectedText()
        cursor.insertBlock()
        self.setTextCursor(cursor)
        self.insertPlainText(text)

    def closeEvent(self, event):
        logger.info("Closing LogViewWidget")
        self._exit_event.set()
        self._logging_queue_listener.close()
        logging.getLogger("").handlers.remove(self._queue_handler)
        super().closeEvent(event)


if __name__ == "__main__":
    import sys

    app = QApplication(sys.argv)
    log_view_widget = LogViewWidget()
    log_view_widget.show()
    sys.exit(app.exec())



import logging
from pathlib import Path
from typing import Union

from PySide6 import QtCore
from PySide6.QtCore import Slot, QFileSystemWatcher

from freemocap.gui.qt.style_sheet.compile_scss_to_css import compile_scss_to_css

logger = logging.getLogger(__name__)


class SCSSFileWatcher(QFileSystemWatcher):
    def __init__(self, path_to_scss_file: Union[str, Path], path_to_css_file: Union[str, Path], parent=None):
        super().__init__(parent)
        self.addPath(str(path_to_scss_file))
        self.parent = parent
        self.path_to_scss_file = str(path_to_scss_file)
        self.path_to_css_file = str(path_to_css_file)
        self.fileChanged.connect(self.file_changed)

    @Slot(str)
    def file_changed(self, path: str):
        logger.info(f"SCSS file changed: {path} - running compile_scss_to_css")

        try:
            compile_scss_to_css(self.path_to_scss_file, self.path_to_css_file)
            logger.info("SCSS compilation finished")
        except Exception as e:
            logger.error(f"Error in SCSS compilation: {e}")


if __name__ == "__main__":
    app = QtCore.QCoreApplication([])
    qt_file_system_watcher = SCSSFileWatcher(
        path_to_scss_file=Path(__file__).parent / "qt_style_sheet.scss",
        path_to_css_file=Path(__file__).parent / "qt_style_sheet.css",
    )
    print(f"watching directories - {qt_file_system_watcher.directories()})")
    print(f"watching files - {qt_file_system_watcher.files()})")
    app.exec()
    print("done!")



from pathlib import Path

def main():
    try:
        from freemocap.data_layer.recording_models.post_processing_parameter_models import ProcessingParameterModel, AniposeTriangulate3DParametersModel, PostProcessingParametersModel
        from freemocap.data_layer.recording_models.recording_info_model import RecordingInfoModel
        from freemocap.core_processes.process_motion_capture_videos.process_recording_folder import process_recording_folder
        from skellytracker.trackers.openpose_tracker.openpose_model_info import OpenPoseModelInfo, OpenPoseTrackingParams
    except ImportError as e:
        if 'ProcessingParameterModel' not in dir() or 'process_folder_of_videos' not in dir() or 'OpenPoseModelInfo' not in dir():
            raise e



    recording_folder = Path(r'D:\mdn_treadmill_for_testing')
    openpose_root_folder_path = r'C:\openpose'


    recording_model = RecordingInfoModel(
        recording_folder_path=recording_folder,
        active_tracker="openpose"
    )

    openpose_processing_parameters = ProcessingParameterModel(
        recording_info_model= recording_model,
        tracking_parameters_model=OpenPoseTrackingParams(
            # net_resolution="-1x656",
            openpose_root_folder_path=str(openpose_root_folder_path),
            run_image_tracking=True
        ),
        anipose_triangulate_3d_parameters_model=AniposeTriangulate3DParametersModel(),
        post_processing_parameters_model=PostProcessingParametersModel(),
        tracking_model_info=OpenPoseModelInfo()
    )

    process_recording_folder(
        recording_processing_parameter_model=openpose_processing_parameters,
        kill_event=None,
        logging_queue=None,
    )

if __name__ == "__main__":
    main()



import logging
import threading

from PySide6.QtCore import Qt, QSize
from PySide6.QtGui import QPixmap
from PySide6.QtWidgets import (
    QDialog,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
    QTabWidget,
    QWidget,
    QScrollArea,  # Added import
)

from freemocap.gui.qt.utilities.save_and_load_gui_state import GuiState, load_gui_state
from freemocap.gui.qt.widgets.release_notes_dialogs.release_notes_content import get_all_release_notes, \
    ReleaseNoteContent
from freemocap.gui.qt.widgets.release_notes_dialogs.release_notes_styles import ReleaseNotesStyles

logger = logging.getLogger(__name__)


class TabbedReleaseNotesDialog(QDialog):
    """
    A tabbed dialog that displays release notes from current and previous versions,
    with options to control when the dialog is shown.
    """

    def __init__(
            self,
            kill_thread_event: threading.Event,
            gui_state: GuiState,
            parent=None,
            min_width: int = 1000,
            min_height: int = 800,
            dark_mode: bool = True,
    ) -> None:
        super().__init__(parent=parent)
        self.kill_thread_event = kill_thread_event
        self.gui_state = gui_state
        self.dark_mode = dark_mode

        self.setMinimumSize(min_width, min_height)
        self.setWindowTitle("FreeMoCap Release Notes")
        self.setStyleSheet(
            ReleaseNotesStyles.get_dialog_style(
                dark_mode=self.dark_mode))

        # Main layout - horizontal to support side tabs
        main_layout = QHBoxLayout()
        main_layout.setContentsMargins(20, 20, 20, 20)
        main_layout.setSpacing(15)
        self.setLayout(main_layout)

        # Create tab widget with tabs on the left side
        self.tab_widget = QTabWidget()
        self.tab_widget.setTabPosition(QTabWidget.South)  # Set tabs to the left side
        self.tab_widget.setStyleSheet(
            ReleaseNotesStyles.get_tab_widget_style(
                dark_mode=self.dark_mode))

        self.tab_widget.tabBar().setElideMode(Qt.ElideNone)

        # Add tabs for different release notes
        self._add_release_notes_tabs()

        main_layout.addWidget(self.tab_widget, 1)

        # Right side panel for options
        # options_panel = self._create_options_panel()
        # main_layout.addWidget(options_panel, 0)  # Fixed width for options panel

    def _add_release_notes_tabs(self) -> None:
        """Add tabs for different release notes."""
        # Get all release notes content
        release_notes: list[ReleaseNoteContent] = get_all_release_notes()

        # Add each release note as a tab
        for release_note in release_notes:
            tab = QWidget()
            layout = QVBoxLayout(tab)
            layout.setContentsMargins(20, 20, 20, 20)

            # Create content widget - now passing image_path
            content_widget = self._create_release_notes_content(release_note)
            layout.addWidget(content_widget)

            # Add tab
            self.tab_widget.addTab(tab, release_note.tab_title)

    def _create_release_notes_content(
            self,
            release_note: ReleaseNoteContent,
    ) -> QWidget:
        """Create a widget with release notes content."""
        content_widget = QWidget()
        content_layout = QHBoxLayout(content_widget)
        content_layout.setContentsMargins(0, 0, 0, 0)
        content_layout.setSpacing(20)

        # Create scroll area for text content
        scroll_area = QScrollArea()
        scroll_area.setWidgetResizable(True)
        scroll_area.setStyleSheet("background-color: transparent;"
                                  "border: none;")

        # Create a container widget for the scrollable content
        scroll_content = QWidget()
        scroll_layout = QVBoxLayout(scroll_content)
        scroll_layout.setContentsMargins(0, 0, 0, 0)

        scroll_layout.setSpacing(15)

        title_widget = QLabel()
        title_widget.setWordWrap(True)
        title_widget.setOpenExternalLinks(True)

        # Title HTML content with inline styles
        title_html_content = f"""
        <html>
        <head>
            <style>
                .title {{
                    font-size: 24px;
                    font-weight: bold;
                    color: {'#ffffff' if self.dark_mode else '#1a1a1a'};
                    margin-bottom: 10px;
                    line-height: 1.3;
                }}
            </style>
        </head>
        <body>
            <div class="title">{release_note.content_title}</div>
        </body>
        </html>
        """
        title_widget.setText(title_html_content)
        scroll_layout.addWidget(title_widget)

        if release_note.content_subtitle:
            subtitle_widget = QLabel()
            subtitle_widget.setWordWrap(True)
            subtitle_widget.setOpenExternalLinks(True)

            # Apply styling directly to the widget
            subtitle_widget.setStyleSheet(f"""
                        QLabel {{
                            font-size: 16px;
                            font-weight: bold;
                            color: {'#4fc3f7' if self.dark_mode else '#2196F3'};
                            padding: 15px 0px 10px 0px;
                        }}
                    """)

            # Simple text content - no HTML styling needed
            subtitle_widget.setText(release_note.content_subtitle)
            scroll_layout.addWidget(subtitle_widget)
        # Add image if provided
        if release_note.image_path:
            from pathlib import Path
            if Path(release_note.image_path).exists():
                image_label = QLabel()
                image_label.setAlignment(Qt.AlignCenter)
                image_pixmap = QPixmap(release_note.image_path)

                if not image_pixmap.isNull():
                    # Scale image to fit within scroll area while maintaining aspect ratio
                    max_width = 540
                    if image_pixmap.width() > max_width:
                        scaled_pixmap = image_pixmap.scaledToWidth(
                            max_width,
                            Qt.SmoothTransformation
                        )
                    else:
                        scaled_pixmap = image_pixmap

                    image_label.setPixmap(scaled_pixmap)

                    # Add some spacing and the image
                    scroll_layout.addSpacing(10)
                    scroll_layout.addWidget(image_label)
                    scroll_layout.addSpacing(10)
                else:
                    logger.warning(f"Failed to load image: {release_note.image_path}")
            else:
                logger.warning(f"Image file does not exist: {release_note.image_path}")

        # Text content
        text_widget = QLabel()
        text_widget.setWordWrap(True)
        text_widget.setOpenExternalLinks(True)
        text_widget.setStyleSheet(
            """
            font-size: 14px;
            line-height: 1.4;
            color: #e0e0e0;
            """
            if self.dark_mode
            else """
            font-size: 14px;
            line-height: 1.4;
            color: #2c3e50;
            """
        )

        # Set HTML content
        html_content = f"""
        <html>
        {ReleaseNotesStyles.get_html_content_style(dark_mode=self.dark_mode)}
        <body>
        {release_note.content_html}
        </body>
        </html>
        """
        text_widget.setText(html_content)
        scroll_layout.addWidget(text_widget)

        # Add stretch to push content to top
        scroll_layout.addStretch()

        # Set the scroll content widget
        scroll_area.setWidget(scroll_content)

        # Add scroll area to layout
        content_layout.addWidget(scroll_area, 1)

        # Logo (on the right side)
        if release_note.logo_path:
            logo_label = QLabel()
            logo_label.setFixedSize(QSize(200, 200))
            logo_label.setAlignment(Qt.AlignCenter)

            pixmap = QPixmap(release_note.logo_path)
            if not pixmap.isNull():
                scaled_pixmap = pixmap.scaled(
                    200, 200, Qt.KeepAspectRatio, Qt.SmoothTransformation
                )
                logo_label.setPixmap(scaled_pixmap)
            else:
                logger.warning(f"Failed to load logo: {release_note.logo_path}")

            content_layout.addWidget(logo_label, 0, Qt.AlignTop)

        return content_widget


if __name__ == "__main__":
    import sys
    from PySide6.QtWidgets import QApplication

    app = QApplication(sys.argv)

    # Show the new tabbed dialog with dark mode enabled
    dialog = TabbedReleaseNotesDialog(
        gui_state=load_gui_state(),
        kill_thread_event=threading.Event(),
        dark_mode=True,  # Set to True for dark mode, False for light mode
    )
    dialog.show()

    sys.exit(app.exec())



from PySide6.QtCore import Qt
from PySide6.QtSvgWidgets import QSvgWidget
from PySide6.QtWidgets import QSizePolicy


class LogoSvgWidget(QSvgWidget):
    def __init__(self, image_path: str):
        super().__init__()
        self.load(image_path)
        self.setSizePolicy(QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Expanding)
        self.renderer().setAspectRatioMode(Qt.AspectRatioMode.KeepAspectRatio)



import logging
from pathlib import Path
from typing import Union, Dict

import numpy as np

from freemocap.system.paths_and_filenames.file_and_folder_names import (
    CENTER_OF_MASS_FOLDER_NAME,
    DATA_2D_NPY_FILE_NAME,
    RAW_3D_NPY_FILE_NAME,
    OUTPUT_DATA_FOLDER_NAME,
    RAW_DATA_FOLDER_NAME,
    TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME,
    REPROJECTION_ERROR_NPY_FILE_NAME,
    SYNCHRONIZED_VIDEOS_FOLDER_NAME,
    ANNOTATED_VIDEOS_FOLDER_NAME,
    DATA_3D_NPY_FILE_NAME,
    OLD_DATA_2D_NPY_FILE_NAME,
    OLD_DATA_3D_NPY_FILE_NAME,
    OLD_RAW_3D_NPY_FILE_NAME,
    OLD_REPROJECTION_ERROR_NPY_FILE_NAME,
    OLD_TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME,
)
from freemocap.system.paths_and_filenames.path_getters import create_camera_calibration_file_name, get_blender_file_path
from freemocap.tests.test_image_tracking_data_shape import (
    test_image_tracking_data_exists,
    test_image_tracking_data_shape,
)
from freemocap.tests.test_skeleton_data_shape import test_skeleton_data_exists, test_skeleton_data_shape
from freemocap.tests.test_synchronized_video_frame_counts import test_synchronized_video_frame_counts
from freemocap.tests.test_total_body_center_of_mass_data_shape import (
    test_total_body_center_of_mass_data_exists,
    test_total_body_center_of_mass_data_shape,
)
from freemocap.utilities.get_number_of_frames_of_videos_in_a_folder import get_number_of_frames_of_videos_in_a_folder
from freemocap.utilities.get_video_paths import get_video_paths

logger = logging.getLogger(__name__)


class RecordingInfoModel:
    def __init__(self, recording_folder_path: Union[Path, str], active_tracker: str = "mediapipe"):
        if any(
            [
                Path(recording_folder_path).name == SYNCHRONIZED_VIDEOS_FOLDER_NAME,
                Path(recording_folder_path).name == ANNOTATED_VIDEOS_FOLDER_NAME,
                Path(recording_folder_path).name == OUTPUT_DATA_FOLDER_NAME,
            ]
        ):
            recording_folder_path = Path(recording_folder_path).parent

        self._path = Path(recording_folder_path)
        self._name = self._path.name

        self._calibration_toml_path = str(
            Path(self._path) / create_camera_calibration_file_name(recording_name=self._name)
        )

        self._active_tracker = active_tracker

        self._recording_folder_status_checker = RecordingFolderStatusChecker(recording_info_model=self)

    @property
    def active_tracker(self) -> str:
        return self._active_tracker

    @active_tracker.setter
    def active_tracker(self, tracker_name: str):
        self._active_tracker = tracker_name

    @property
    def file_prefix(self) -> str:
        if self.active_tracker != "" and self.active_tracker[-1] != "_":
            return self.active_tracker + "_"
        else:
            return self.active_tracker

    # TODO: Create setters for the path names that depend on tracker name, and an overall setter that sets them all at once
    @property
    def path(self) -> str:
        return str(self._path)

    @property
    def name(self) -> str:
        return self._name

    @property
    def status_check(self) -> Dict[str, Union[bool, str, float, dict]]:
        return self._recording_folder_status_checker.status_check

    @property
    def calibration_toml_path(self) -> str:
        return self._calibration_toml_path

    @calibration_toml_path.setter
    def calibration_toml_path(self, path: Union[Path, str]):
        self._calibration_toml_path = str(path)

    @property
    def output_data_folder_path(self) -> str:
        return str(Path(self._path) / OUTPUT_DATA_FOLDER_NAME)

    @property
    def raw_data_folder_path(self) -> str:
        return str(Path(self.output_data_folder_path) / RAW_DATA_FOLDER_NAME)

    @property
    def synchronized_videos_folder_path(self) -> str:
        return str(Path(self._path) / SYNCHRONIZED_VIDEOS_FOLDER_NAME)

    @property
    def annotated_videos_folder_path(self) -> str:
        return str(Path(self._path) / ANNOTATED_VIDEOS_FOLDER_NAME)

    @property
    def data_2d_npy_file_path(self):
        data_2d_path = Path(self.raw_data_folder_path) / (self.file_prefix + DATA_2D_NPY_FILE_NAME)
        old_data_2d_path = Path(self.raw_data_folder_path) / OLD_DATA_2D_NPY_FILE_NAME
        if self.active_tracker == "mediapipe" and old_data_2d_path.exists() and not data_2d_path.exists():
            return str(old_data_2d_path)
        else:
            return str(data_2d_path)

    @property
    def data_3d_npy_file_path(self):
        data_3d_path = Path(self._path) / OUTPUT_DATA_FOLDER_NAME / (self.file_prefix + DATA_3D_NPY_FILE_NAME)
        old_data_3d_path = Path(self._path) / OUTPUT_DATA_FOLDER_NAME / OLD_DATA_3D_NPY_FILE_NAME
        if self.active_tracker == "mediapipe" and old_data_3d_path.exists() and not data_3d_path.exists():
            return str(old_data_3d_path)
        else:
            return str(data_3d_path)

    @property
    def raw_data_3d_npy_file_path(self):
        raw_data_path = (
            Path(self._path)
            / OUTPUT_DATA_FOLDER_NAME
            / RAW_DATA_FOLDER_NAME
            / (self.file_prefix + RAW_3D_NPY_FILE_NAME)
        )
        old_raw_data_path = Path(self._path) / OUTPUT_DATA_FOLDER_NAME / RAW_DATA_FOLDER_NAME / OLD_RAW_3D_NPY_FILE_NAME
        if self.active_tracker == "mediapipe" and old_raw_data_path.exists() and not raw_data_path.exists():
            return str(old_raw_data_path)
        else:
            return str(raw_data_path)

    @property
    def reprojection_error_data_npy_file_path(self):
        reprojection_error_path = (
            Path(self._path)
            / OUTPUT_DATA_FOLDER_NAME
            / RAW_DATA_FOLDER_NAME
            / (self.file_prefix + REPROJECTION_ERROR_NPY_FILE_NAME)
        )
        raw_reprojection_error_path = (
            Path(self._path) / OUTPUT_DATA_FOLDER_NAME / RAW_DATA_FOLDER_NAME / OLD_REPROJECTION_ERROR_NPY_FILE_NAME
        )
        if (
            self.active_tracker == "mediapipe"
            and raw_reprojection_error_path.exists()
            and not reprojection_error_path.exists()
        ):
            return str(raw_reprojection_error_path)
        else:
            return str(reprojection_error_path)

    @property
    def total_body_center_of_mass_npy_file_path(self):
        total_body_com_path = (
            Path(self._path)
            / OUTPUT_DATA_FOLDER_NAME
            / CENTER_OF_MASS_FOLDER_NAME
            / (self.file_prefix + TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME)
        )
        raw_total_body_com_path = (
            Path(self._path)
            / OUTPUT_DATA_FOLDER_NAME
            / CENTER_OF_MASS_FOLDER_NAME
            / OLD_TOTAL_BODY_CENTER_OF_MASS_NPY_FILE_NAME
        )
        if self.active_tracker == "mediapipe" and raw_total_body_com_path.exists() and not total_body_com_path.exists():
            return str(raw_total_body_com_path)
        else:
            return str(total_body_com_path)

    @property
    def blender_file_path(self):
        return get_blender_file_path(str(self._path))

    @property
    def blender_file_status_check(self):
        return Path(self.blender_file_path).is_file()

    @property
    def calibration_toml_check(self) -> bool:
        return self._recording_folder_status_checker.check_calibration_toml_status()

    @property
    def synchronized_videos_status_check(self) -> bool:
        return self._recording_folder_status_checker.check_synchronized_videos_status()

    @property
    def single_video_check(self) -> bool:
        return self._recording_folder_status_checker.check_single_video()

    @property
    def data2d_status_check(self) -> bool:
        return self._recording_folder_status_checker.check_data2d_status()

    @property
    def data3d_status_check(self) -> bool:
        return self._recording_folder_status_checker.check_data3d_status()

    @property
    def center_of_mass_data_status_check(self) -> bool:
        return self._recording_folder_status_checker.check_center_of_mass_data_status()


class RecordingFolderStatusChecker:
    def __init__(self, recording_info_model: RecordingInfoModel):
        self.recording_info_model = recording_info_model

    @property
    def status_check(self) -> Dict[str, Union[bool, str, float, dict]]:
        return {
            "synchronized_videos_status_check": self.check_synchronized_videos_status(),
            "data2d_status_check": self.check_data2d_status(),
            "data3d_status_check": self.check_data3d_status(),
            "center_of_mass_data_status_check": self.check_center_of_mass_data_status(),
            "blender_file_status_check": self.check_blender_file_status(),
            "single_video_check": self.check_single_video(),
            "calibration_toml_check": self.check_calibration_toml_status(),
            "video_and_camera_info": {
                "number_of_synchronized_videos": self.get_number_of_mp4s_in_synched_videos_directory(),
                "number_of_frames_in_videos": self.get_number_of_frames_in_videos(),
                # 'camera_rotation_and_translation': self.get_camera_rotation_and_translation_from_calibration_toml()
            },
        }

    def check_synchronized_videos_status(self) -> bool:
        try:
            test_synchronized_video_frame_counts(self.recording_info_model.synchronized_videos_folder_path)
            return True
        except AssertionError:
            return False

    def check_single_video(self) -> bool:
        if len(get_video_paths(path_to_video_folder=self.recording_info_model.synchronized_videos_folder_path)) == 1:
            return True
        else:
            return False

    def check_data2d_status(self) -> bool:
        try:
            test_image_tracking_data_exists(
                synchronized_video_folder_path=self.recording_info_model.synchronized_videos_folder_path,
                image_tracking_data=self.recording_info_model.data_2d_npy_file_path,
            )
            test_image_tracking_data_shape(
                synchronized_video_folder_path=self.recording_info_model.synchronized_videos_folder_path,
                image_tracking_data=self.recording_info_model.data_2d_npy_file_path,
            )
            return True
        except AssertionError:
            return False

    def check_data3d_status(self) -> bool:
        try:
            test_skeleton_data_exists(
                raw_skeleton_data=self.recording_info_model.data_3d_npy_file_path,
                reprojection_error_data=self.recording_info_model.reprojection_error_data_npy_file_path,
            )
            test_skeleton_data_shape(
                synchronized_video_folder_path=self.recording_info_model.synchronized_videos_folder_path,
                raw_skeleton_data=self.recording_info_model.data_3d_npy_file_path,
                reprojection_error_data=self.recording_info_model.reprojection_error_data_npy_file_path,
            )
            return True
        except AssertionError:
            return False

    def check_center_of_mass_data_status(self) -> bool:
        try:
            test_total_body_center_of_mass_data_exists(
                total_body_center_of_mass_data=self.recording_info_model.total_body_center_of_mass_npy_file_path,
            )
            test_total_body_center_of_mass_data_shape(
                synchronized_video_folder_path=self.recording_info_model.synchronized_videos_folder_path,
                total_body_center_of_mass_data=self.recording_info_model.total_body_center_of_mass_npy_file_path,
            )
            return True
        except AssertionError:
            return False

    def check_blender_file_status(self) -> bool:
        return Path(self.recording_info_model.blender_file_path).is_file()

    def check_calibration_toml_status(self) -> bool:
        # TODO: check if adding a single video check here makes sense with the logic (i.e. will that ever give a misleading response)
        toml_status = Path(self.recording_info_model.calibration_toml_path).is_file()
        if not toml_status:
            logger.debug(
                "No calibration file found with session name, checking for other calibration files in recording session"
            )
            toml_status = self.check_for_calibration_toml_with_different_name()
        return toml_status

    def get_number_of_mp4s_in_synched_videos_directory(self) -> int:
        synchronized_directory_path = Path(self.recording_info_model.synchronized_videos_folder_path)

        if not synchronized_directory_path.exists():
            return 0

        video_count = len(get_video_paths(synchronized_directory_path))

        logger.info(f"Number of `.mp4`'s in {synchronized_directory_path}: {video_count}")
        return video_count

    def get_number_of_frames_in_videos(self) -> Dict[str, int]:
        timestamps_directory_path = Path(self.recording_info_model.synchronized_videos_folder_path) / "timestamps"

        if timestamps_directory_path.exists():
            frame_counts = {}

            for npy_file in timestamps_directory_path.iterdir():
                if npy_file.is_file() and npy_file.suffix.lower() == ".npy":
                    video_npy = np.load(str(npy_file))
                    frame_counts[npy_file.name] = len(video_npy) - 1
        else:
            logger.debug("No 'timestamps' directory, finding frame counts from synchronized videos")
            try:
                frame_counts = get_number_of_frames_of_videos_in_a_folder(
                    self.recording_info_model.synchronized_videos_folder_path
                )
            except ValueError:
                frame_counts = {}

        return frame_counts

    def check_for_calibration_toml_with_different_name(self) -> bool:
        try:
            for file in Path(self.recording_info_model.path).iterdir():
                if file.is_file() and file.name.endswith("camera_calibration.toml"):
                    self.recording_info_model.calibration_toml_path = str(file)
                    logger.info(f"Found calibration file at: {self.recording_info_model.calibration_toml_path}")
                    return True
        except Exception as e:
            logger.warning(f"Error checking for calibration toml with different name: {e}")
        return False



"""
Stylesheet definitions for the release notes dialogs.
Provides both light and dark mode styles.
"""


class ReleaseNotesStyles:
    """
    Provides stylesheets for the release notes dialogs in both light and dark modes.
    """

    @staticmethod
    def get_dialog_style(dark_mode: bool = True) -> str:
        """Return the CSS style for the dialog."""
        if dark_mode:
            return """
                QDialog {
                    background-color: #1e1e1e;
                }
                QLabel {
                    color: #e0e0e0;
                }
                QRadioButton {
                    color: #c0c0c0;
                    font-size: 13px;
                    spacing: 8px;
                }
                QRadioButton::indicator {
                    width: 16px;
                    height: 16px;
                }
            """
        else:
            return """
                QDialog {
                    background-color: #f8f9fa;
                }
                QLabel {
                    color: #212529;
                }
                QRadioButton {
                    color: #495057;
                    font-size: 13px;
                    spacing: 8px;
                }
                QRadioButton::indicator {
                    width: 16px;
                    height: 16px;
                }
            """

    @staticmethod
    def get_tab_widget_style(dark_mode: bool = True) -> str:
        """Return the CSS style for the tab widget."""
        if dark_mode:
            return """
                QTabWidget::pane {
                    border: 1px solid #3a3a3a;
                    border-radius: 5px;
                    background-color: #2d2d2d;
                }
                QTabBar::tab {
                    background-color: #383838;
                    border: 1px solid #3a3a3a;
                    border-right: none;
                    border-top-left-radius: 4px;
                    border-bottom-left-radius: 4px;
                    padding: 10px 15px;
                    margin-bottom: 2px;
                    min-height: 25px;
                    min-width: 150px;
                    text-align: left;
                    color: #c0c0c0;
                }
                QTabBar::tab:selected {
                    background-color: #2d2d2d;
                    border-right: 1px solid #2d2d2d;
                    color: #ffffff;
                    font-weight: bold;
                    border-top: 3px solid #3498db;  /* Blue accent border */
                    border-left: 1px solid #3498db;  /* Blue accent border */
                    border-right: 1px solid #3498db;  /* Blue accent border */
                    border-bottom: 1px solid #3498db;  /* Blue accent border */
                }
                QTabBar::tab:hover:!selected {
                    background-color: #454545;
                }
                /* This is the key part for horizontal text in vertical tabs */
                QTabBar::tab:left {
                    margin-right: 2px;
                }
            """
        else:
            return """
                QTabWidget::pane {
                    border: 1px solid #d0d0d0;
                    border-radius: 5px;
                    background-color: #ffffff;
                }
                QTabBar::tab {
                    background-color: #f0f0f0;
                    border: 1px solid #d0d0d0;
                    border-right: none;
                    border-top-left-radius: 4px;
                    border-bottom-left-radius: 4px;
                    padding: 10px 15px;
                    margin-bottom: 2px;
                    min-height: 25px;
                    min-width: 150px;
                    text-align: left;
                }
                QTabBar::tab:selected {
                    background-color: #ffffff;
                    border-right: 1px solid #ffffff;
                    font-weight: bold;
                    border-left: 3px solid #3498db;  /* Blue accent border */
                }
                QTabBar::tab:hover:!selected {
                    background-color: #e0e0e0;
                }
                /* This is the key part for horizontal text in vertical tabs */
                QTabBar::tab:left {
                    margin-right: 2px;
                }
            """

    @staticmethod
    def get_group_box_style(dark_mode: bool = True) -> str:
        """Return the CSS style for group boxes."""
        if dark_mode:
            return """
                QGroupBox {
                    font-weight: bold;
                    border: 1px solid #3a3a3a;
                    border-radius: 5px;
                    margin-top: 10px;
                    padding-top: 15px;
                    color: #e0e0e0;
                }
                QGroupBox::title {
                    subcontrol-origin: margin;
                    left: 10px;
                    padding: 0 5px;
                }
            """
        else:
            return """
                QGroupBox {
                    font-weight: bold;
                    border: 1px solid #d0d0d0;
                    border-radius: 5px;
                    margin-top: 10px;
                    padding-top: 15px;
                }
                QGroupBox::title {
                    subcontrol-origin: margin;
                    left: 10px;
                    padding: 0 5px;
                }
            """

    @staticmethod
    def get_button_style(dark_mode: bool = True) -> str:
        """Return the CSS style for buttons."""
        if dark_mode:
            return """
                QPushButton {
                    background-color: #2980b9;
                    color: white;
                    border-radius: 5px;
                    border: 1px solid #2980b9;
                    padding: 10px 25px;
                    font-weight: bold;
                    min-width: 120px;
                }
                QPushButton:hover {
                    background-color: #3498db;
                }
            """
        else:
            return """
                QPushButton {
                    background-color: #3498db;
                    color: white;
                    border-radius: 5px;
                    border: 1px solid #3498db;
                    padding: 10px 25px;
                    font-weight: bold;
                    min-width: 120px;
                }
                QPushButton:hover {
                    background-color: #2980b9;
                }
            """

    @staticmethod
    def get_html_content_style(dark_mode: bool = True) -> str:
        """Return the CSS style for HTML content."""
        if dark_mode:
            return """
            <style>
            body { color: #e0e0e0; }
            a { color: #5dade2; text-decoration: none; font-weight: 500; }
            .feature { color: #2ecc71; font-weight: bold; }
            </style>
            """
        else:
            return """
            <style>
            a { color: #2980b9; text-decoration: none; font-weight: 500; }
            .feature { color: #27ae60; font-weight: bold; }
            </style>
            """



import logging
from pathlib import Path
from typing import Union

import numpy as np

from freemocap.system.paths_and_filenames.file_and_folder_names import (
    RAW_3D_NPY_FILE_NAME,
    FULL_REPROJECTION_ERROR_NPY_FILE_NAME,
    REPROJECTION_ERROR_NPY_FILE_NAME,
    REPROJECTION_FILTERED_PREFIX,
)

logger = logging.getLogger(__name__)


def save_3d_data_to_npy(
    data3d_numFrames_numTrackedPoints_XYZ: np.ndarray,
    data3d_numFrames_numTrackedPoints_reprojectionError: np.ndarray,
    data3d_numCams_numFrames_numTrackedPoints_reprojectionError: np.ndarray,
    path_to_folder_where_data_will_be_saved: Union[str, Path],
    processing_level: str,
    file_prefix: str = "",
):
    path_to_folder_where_data_will_be_saved = Path(path_to_folder_where_data_will_be_saved)
    Path(path_to_folder_where_data_will_be_saved).mkdir(parents=True, exist_ok=True)  # save spatial XYZ data
    if processing_level == "raw":
        data_3d_save_path = path_to_folder_where_data_will_be_saved / f"{file_prefix}_{RAW_3D_NPY_FILE_NAME}"
        reprojection_error_save_path = (
            path_to_folder_where_data_will_be_saved / f"{file_prefix}_{REPROJECTION_ERROR_NPY_FILE_NAME}"
        )
        full_reprojection_error_save_path = (
            path_to_folder_where_data_will_be_saved / f"{file_prefix}_{FULL_REPROJECTION_ERROR_NPY_FILE_NAME}"
        )
    elif processing_level == "reprojection_filtered":
        data_3d_save_path = (
            path_to_folder_where_data_will_be_saved
            / f"{REPROJECTION_FILTERED_PREFIX}{file_prefix}_{RAW_3D_NPY_FILE_NAME}"
        )
        reprojection_error_save_path = (
            path_to_folder_where_data_will_be_saved
            / f"{REPROJECTION_FILTERED_PREFIX}{file_prefix}_{REPROJECTION_ERROR_NPY_FILE_NAME}"
        )
        full_reprojection_error_save_path = (
            path_to_folder_where_data_will_be_saved
            / f"{REPROJECTION_FILTERED_PREFIX}{file_prefix}_{FULL_REPROJECTION_ERROR_NPY_FILE_NAME}"
        )
    else:
        logger.exception(f"processing_level: {processing_level} not recognized")
        raise Exception

    logger.info(f"saving: {data_3d_save_path}")
    np.save(str(data_3d_save_path), data3d_numFrames_numTrackedPoints_XYZ)

    # save reprojection error

    logger.info(f"saving: {reprojection_error_save_path}")
    np.save(
        str(reprojection_error_save_path),
        data3d_numFrames_numTrackedPoints_reprojectionError,
    )

    logger.info(f"saving: {full_reprojection_error_save_path}")
    np.save(
        str(full_reprojection_error_save_path),
        data3d_numCams_numFrames_numTrackedPoints_reprojectionError,
    )



import json
from pathlib import Path

from pydantic import BaseModel

from freemocap.core_processes.capture_volume_calibration.charuco_stuff.charuco_board_definition import CHARUCO_BOARDS
from freemocap.core_processes.export_data.blender_stuff.get_best_guess_of_blender_path import (
    get_best_guess_of_blender_path,
)
from freemocap.system.paths_and_filenames.file_and_folder_names import BASE_FREEMOCAP_DATA_FOLDER_NAME


class GuiState(BaseModel):
    first_run: bool = False
    send_user_pings: bool = True
    show_welcome_screen: bool = True
    auto_process_videos_on_save: bool = True
    generate_jupyter_notebook: bool = True
    auto_open_in_blender: bool = True
    charuco_square_size: float = 39
    use_charuco_as_groundplane: bool = False
    annotate_charuco_images: bool = False
    freemocap_data_folder_path: str = str(Path(Path.home(), BASE_FREEMOCAP_DATA_FOLDER_NAME))
    blender_path: str = str(get_best_guess_of_blender_path())
    charuco_board_name: str = next(iter(CHARUCO_BOARDS))
    show_data_quality_warning: bool = False
    shown_latest_release_notes: bool = False


def save_gui_state(gui_state: GuiState, file_pathstring: str | None = None) -> None:
    from freemocap.system.paths_and_filenames.path_getters import get_gui_state_json_path

    if file_pathstring is None:
        file_pathstring = get_gui_state_json_path()
    with open(file_pathstring, "w") as file:
        json.dump(gui_state.model_dump(mode="json"), file, indent=4)


def load_gui_state(file_pathstring: str | None = None) -> GuiState:
    if file_pathstring is None:
        from freemocap.system.paths_and_filenames.path_getters import get_gui_state_json_path

        file_pathstring = get_gui_state_json_path()
    try:
        with open(file_pathstring, "r") as file:
            gui_state = GuiState(**json.load(file))
    except OSError:
        gui_state = GuiState(first_run=True)
    return gui_state



import filecmp
import logging
import shutil
from pathlib import Path
from typing import Union

from freemocap.core_processes.capture_volume_calibration.anipose_camera_calibration import (
    freemocap_anipose,
)
from freemocap.system.paths_and_filenames.path_getters import get_last_successful_calibration_toml_path

logger = logging.getLogger(__name__)


def load_most_recent_anipose_calibration_toml(
        save_copy_of_calibration_to_this_path: Union[str, Path, None] = None
) -> freemocap_anipose.CameraGroup:
    most_recent_calibration_toml_path = get_last_successful_calibration_toml_path()
    logger.info(f"loading `most recent calibration from:{str(most_recent_calibration_toml_path)}")
    if save_copy_of_calibration_to_this_path is not None:
        logger.info(f"Saving copy of `most_recent_calibration.toml` to {save_copy_of_calibration_to_this_path}")

        shutil.copy(
            str(most_recent_calibration_toml_path),
            str(Path(save_copy_of_calibration_to_this_path) / Path(most_recent_calibration_toml_path).name),
        )

    try:
        camera_group = freemocap_anipose.CameraGroup.load(str(most_recent_calibration_toml_path))
    except Exception as e:
        logger.exception(e)
        raise
    return camera_group


def load_anipose_calibration_toml_from_path(
        camera_calibration_data_toml_path: Union[str, Path],
        save_copy_of_calibration_to_this_path: Union[str, Path, None] = None,
) -> freemocap_anipose.CameraGroup:
    logger.info(f"loading camera calibration file from:{str(camera_calibration_data_toml_path)}")
    try:
        anipose_calibration_object = freemocap_anipose.CameraGroup.load(str(camera_calibration_data_toml_path))
        if save_copy_of_calibration_to_this_path is not None:
            copy_toml_path = str(
                Path(save_copy_of_calibration_to_this_path) / Path(camera_calibration_data_toml_path).name
            )

            if Path(copy_toml_path).is_file() and not filecmp.cmp(
                    str(camera_calibration_data_toml_path), copy_toml_path
            ):
                logger.info(
                    f"Saving copy of {camera_calibration_data_toml_path} to {save_copy_of_calibration_to_this_path}"
                )
                shutil.copyfile(str(camera_calibration_data_toml_path), copy_toml_path)

        return anipose_calibration_object
    except Exception as e:
        logger.error(f"Failed to load anipose calibration info from {str(camera_calibration_data_toml_path)}")
        raise e


def load_calibration_from_session_id(
        session_calibration_file_path: Union[str, Path],
) -> freemocap_anipose.CameraGroup:
    logger.info(f"loading camera calibration file from:{str(session_calibration_file_path)}")
    try:
        return freemocap_anipose.CameraGroup.load(str(session_calibration_file_path))
    except Exception as e:
        logger.error(f"Failed to load anipose calibration info from {str(session_calibration_file_path)}")
        raise e



import nox
from nox.sessions import Session


@nox.session(python=["3.10", "3.11", "3.12"])
def test(session: Session):
    """Run the full test suite on python versions 3.10, 3.11, and 3.12."""
    session.install("-e", ".")
    session.run("pytest", "freemocap/tests")


@nox.session(python=["3.10"])
def test_10(session: Session):
    """Run the full test suite on python versions 3.10."""
    session.install("-e", ".")
    session.run("pytest", "freemocap/tests")


@nox.session(python=["3.11", "3.12"])
def test_11_12(session: Session):
    """Run the full test suite on python versions 3.11 and 3.12."""
    session.install("-e", ".")
    session.run("pytest", "freemocap/tests")


@nox.session(python="3.12")
def coverage(session: Session):
    """Run a coverage test on python 3.12."""
    session.install("-e", ".", "pytest-cov")
    session.run("pytest", "--cov=freemocap", "freemocap/tests")


@nox.session(python="3.12")
def lint(session: Session):
    """Lint using Flake8"""
    session.install(
        "flake8",
        "flake8-bandit",
        "flake8-bugbear",
    )
    session.run("flake8")



from dataclasses import dataclass
from typing import Dict

import cv2


@dataclass
class CharucoBoardDefinition:
    name: str
    number_of_squares_width: int
    number_of_squares_height: int
    black_square_side_length: int
    aruco_marker_length_proportional: float
    aruco_marker_dict: Dict = cv2.aruco.getPredefinedDictionary(cv2.aruco.DICT_4X4_250)

    def __post_init__(self):
        self.charuco_board = cv2.aruco.CharucoBoard(
            size=[self.number_of_squares_width, self.number_of_squares_height],
            squareLength=self.black_square_side_length,
            markerLength=self.aruco_marker_length_proportional,
            dictionary=self.aruco_marker_dict,
        )

        self.number_of_charuco_corners = (self.number_of_squares_width - 1) * (self.number_of_squares_height - 1)


def charuco_7x5() -> CharucoBoardDefinition:
    return CharucoBoardDefinition(
        name="7x5 Charuco",
        number_of_squares_width=7,
        number_of_squares_height=5,
        black_square_side_length=1,
        aruco_marker_length_proportional=0.8,
    )


def charuco_5x3() -> CharucoBoardDefinition:
    return CharucoBoardDefinition(
        name="5x3 Charuco",
        number_of_squares_width=5,
        number_of_squares_height=3,
        black_square_side_length=1,
        aruco_marker_length_proportional=0.8,
    )


CHARUCO_BOARD_IMAGES_URL = "https://github.com/freemocap/freemocap/blob/main/freemocap/assets/charuco/"
CHARUCO_BOARDS = {
    "7x5 Charuco": charuco_7x5,
    "5x3 Charuco": charuco_5x3,
}



from pathlib import Path

import pandas as pd
import plotly.graph_objects as go
import plotly.io as pio
from jinja2 import Template
from packaging.version import Version
from packaging.version import parse as vparse
from plotly.subplots import make_subplots

CURRENT_SENTINEL = Version("9999.0.0")
EXPECTED = 58.0
OS_ORDER = ["Windows", "macOS", "Linux"]


def safe_parse(ver: str) -> Version:
    """Parse semantic versions; return a giant sentinel for 'current'"""
    return CURRENT_SENTINEL if ver == "current" else vparse(ver)


def load_summary_data():
    summary_csv = Path("freemocap/diagnostics/diagnostic_data/calibration_diagnostics_summary.csv")

    print(f"Loading data from: {summary_csv}")
    df = pd.read_csv(summary_csv)

    # Standardize OS names - remove any whitespace and fix case
    df["os"] = df["os"].str.strip()

    # Add version_key for sorting
    df["version_key"] = df["version"].apply(safe_parse)

    # Calculate mean_error if not present
    if "mean_error" not in df.columns:
        df["mean_error"] = df["mean_distance"] - EXPECTED

    # Sort by OS and version
    df = df.sort_values(["os", "version_key"], ascending=[True, True])

    return df


def generate_figures(df):
    OS_COLORS = {
        "Windows": "rgb(0, 114, 178)",  # blue
        "macOS": "rgb(213, 94, 0)",  # vermilion
        "Linux": "rgb(0, 158, 115)",  # bluish green
    }
    # Figure 1 – All OS mean distance over all versions
    fig1 = go.Figure()

    print("\n=== FIGURE 1 DATA ===")

    for os_name in OS_ORDER:
        # Try exact match first
        os_df = df[df["os"] == os_name].sort_values("version_key")

        print(f"\n{os_name}: {len(os_df)} data points")

        if len(os_df) > 0:
            print(f"  Versions: {list(os_df['version'])}")
            print(f"  Values: {list(os_df['mean_distance'])}")

            fig1.add_scatter(
                x=os_df["version"],
                y=os_df["mean_distance"],
                mode="lines+markers",
                name=os_name,
                line=dict(width=2),
                marker=dict(size=8, color=OS_COLORS.get(os_name, "gray")),
            )

    fig1.add_hline(
        y=EXPECTED,
        line_dash="dash",
        line_color="black",
        annotation_text="Expected size",
        annotation_position="top right",
    )

    fig1.update_layout(
        title="Mean Charuco Square Size (mm) – all operating systems",
        yaxis_title="Square Size Estimate (mm)",
        xaxis_title="FreeMoCap Version",
        title_font=dict(size=20),
        xaxis_title_font=dict(size=22),
        yaxis_title_font=dict(size=20),
        xaxis_tickfont=dict(size=16),
        yaxis_tickfont=dict(size=18),
        height=500,
        showlegend=True,
        legend=dict(x=0.02, y=0.98, xanchor="left", yanchor="top"),
    )

    # Figure 2 – Per OS, post-1.6.0
    print("\n=== FIGURE 2 DATA ===")

    # Filter for versions >= 1.6.0 (including "current")
    post = df[(df["version_key"] >= vparse("1.6.0")) | (df["version"] == "current")]
    print(f"Post-1.6.0 data: {len(post)} rows")
    print(f"OS distribution: {post['os'].value_counts().to_dict()}")

    fig2 = make_subplots(rows=1, cols=3, shared_yaxes=True, subplot_titles=OS_ORDER, horizontal_spacing=0.1)

    for col, os_name in enumerate(OS_ORDER, start=1):
        # Get data for this OS and sort by version
        os_data = post[post["os"] == os_name].sort_values("version_key")

        print(f"\n{os_name} (subplot {col}): {len(os_data)} points")
        if len(os_data) > 0:
            print(f"  Versions: {list(os_data['version'])}")
            print(f"  Values: {list(os_data['mean_distance'])}")

        if len(os_data) > 0:
            # Add scatter plot with error bars
            fig2.add_scatter(
                x=os_data["version"],
                y=os_data["mean_distance"],
                error_y=dict(type="data", array=os_data["std_distance"], visible=True, width=4, thickness=2),
                mode="markers",
                marker=dict(size=10, color=OS_COLORS.get(os_name, "gray")),
                showlegend=False,
                row=1,
                col=col,
            )

            # Add value annotations
            for _, row in os_data.iterrows():
                fig2.add_annotation(
                    x=row["version"],
                    y=row["mean_distance"] + row["std_distance"] + 0.3,
                    text=f"{row['mean_distance']:.2f}±{row['std_distance']:.2f}",
                    showarrow=False,
                    yanchor="bottom",
                    row=1,
                    col=col,
                    font=dict(size=10),
                )

        # Add expected line
        fig2.add_hline(y=EXPECTED, line_dash="dash", line_color="black", row=1, col=col)

        # Update x-axis
        fig2.update_xaxes(tickfont=dict(size=14), title_font=dict(size=16), title_text="Version", row=1, col=col)

    # Update y-axis (only for first subplot)
    fig2.update_yaxes(
        tickfont=dict(size=14),
        title_font=dict(size=16),
        title_text="Square-size estimate (mm)",
        range=[EXPECTED - 3, EXPECTED + 3],
        row=1,
        col=1,
    )

    fig2.update_layout(title="Charuco Square Size Estimate – versions ≥ 1.6.0", title_font=dict(size=20), height=400)

    # Figure 3 – Mean error plot
    print("\n=== FIGURE 3 DATA ===")

    fig3 = go.Figure()

    for os_name in OS_ORDER:
        os_data = post[post["os"] == os_name].sort_values("version_key")

        print(f"\n{os_name}: {len(os_data)} points")
        if len(os_data) > 0:
            print(f"  Versions: {list(os_data['version'])}")
            print(f"  Errors: {list(os_data['mean_error'])}")

        if len(os_data) > 0:
            fig3.add_trace(
                go.Scatter(
                    x=os_data["version"],
                    y=os_data["mean_error"],
                    mode="lines+markers",
                    name=os_name,
                    line=dict(width=2),
                    marker=dict(size=8, color=OS_COLORS.get(os_name, "gray")),
                )
            )

    fig3.add_hline(y=0, line_dash="dot", line_color="gray", annotation_text="No error", annotation_position="top right")

    fig3.update_layout(
        title="Mean Error in Square Size Estimate (Post v1.6.0)",
        yaxis_title="Mean error (mm)",
        xaxis_title="FreeMoCap version",
        height=400,
        xaxis_title_font=dict(size=22),
        yaxis_title_font=dict(size=20),
        xaxis_tickfont=dict(size=16),
        yaxis_tickfont=dict(size=18),
        showlegend=True,
        legend=dict(x=0.02, y=0.98, xanchor="left", yanchor="top"),
    )
    fig3.update_yaxes(
        range=[-0.5, 0.5],
    )
    return fig1, fig2, fig3


def generate_summary_table(df):
    print("\n=== TABLE DATA ===")

    # Get the latest data for each OS (highest version_key)
    latest = df.sort_values("version_key", ascending=False).groupby("os").first().reset_index()

    print("Latest data per OS:")
    print(latest[["os", "version", "mean_distance", "std_distance", "mean_error"]].to_string())

    # Create ordered dataframe
    ordered_latest = pd.DataFrame()
    for os_name in OS_ORDER:
        os_row = latest[latest["os"] == os_name]
        if len(os_row) > 0:
            ordered_latest = pd.concat([ordered_latest, os_row])

    if len(ordered_latest) == 0:
        print("WARNING: No data for table!")
        ordered_latest = pd.DataFrame(
            {"os": OS_ORDER, "mean_distance": [0, 0, 0], "std_distance": [0, 0, 0], "mean_error": [0, 0, 0]}
        )

    table = go.Figure(
        data=[
            go.Table(
                header=dict(
                    values=["OS", "Mean Square Size ± SD (mm)", "Mean Error (mm)"],
                    fill_color="lightgray",
                    align="center",
                    font=dict(size=18),
                ),
                cells=dict(
                    values=[
                        ordered_latest["os"],
                        [
                            f"{m:.2f} ± {s:.2f}"
                            for m, s in zip(ordered_latest["mean_distance"], ordered_latest["std_distance"])
                        ],
                        [f"{e:.2f}" for e in ordered_latest["mean_error"]],
                    ],
                    align="center",
                    font=dict(size=16),
                    height=30,
                    fill_color="#f8f9fa",
                ),
            )
        ]
    )

    table.update_layout(title="Latest Calibration Summary (per OS)", margin=dict(t=60, l=0, r=0), height=250)

    return table


def generate_html_report(df, output_path="freemocap/diagnostics/calibration_diagnostics.html"):
    fig1, fig2, fig3 = generate_figures(df)
    table = generate_summary_table(df)

    template = Template(
        """
    <!DOCTYPE html>
    <html>
    <head>
        <meta charset='utf-8'>
        <title>Calibration Diagnostics Report</title>
        <script src='https://cdn.plot.ly/plotly-latest.min.js'></script>
        <style>
            body { font-family: Arial, sans-serif; margin: 40px; }
            h1 { color: #333; }
            .plot-container { margin: 20px 0; }
            pre { background: #f0f0f0; padding: 10px; overflow-x: auto; }
        </style>
    </head>
    <body>
        <h1>Calibration Diagnostics Report</h1>

        <hr><h2>Latest Calibration Summary (per OS)</h2>
        <p>Expected square size: <strong>{{ expected }} mm</strong></p>
        <div style='max-width:1200px; margin:auto;'>{{ table|safe }}</div>

        <hr><h2>Mean Charuco Square Size Per OS</h2>
        <div class="plot-container">{{ fig1|safe }}</div>
        <p style='font-size:0.9em;'>Dashed line = expected size.</p>

        <hr><h2>Mean Charuco Square Size – versions ≥ 1.6.0</h2>
        <div class="plot-container">{{ fig2|safe }}</div>
        <p style='font-size:0.9em;'>Error bars show ±1 SD; numerical values annotated.</p>

        <hr><h2>Mean Square Size Error – versions ≥ 1.6.0</h2>
        <div class="plot-container">{{ fig3|safe }}</div>
    </body>
    </html>
    """
    )

    # Capture debug info
    import io
    import sys

    old_stdout = sys.stdout
    sys.stdout = buffer = io.StringIO()

    # Re-run data loading to capture debug output
    _ = load_summary_data()

    debug_output = buffer.getvalue()
    sys.stdout = old_stdout

    rendered = template.render(
        fig1=pio.to_html(fig1, include_plotlyjs=False, full_html=False),
        fig2=pio.to_html(fig2, include_plotlyjs=False, full_html=False),
        fig3=pio.to_html(fig3, include_plotlyjs=False, full_html=False),
        table=pio.to_html(table, include_plotlyjs=False, full_html=False),
        expected=EXPECTED,
        debug_info=debug_output,
    )

    output_file = Path(output_path)
    output_file.parent.mkdir(parents=True, exist_ok=True)
    output_file.write_text(rendered, encoding="utf-8")
    print(f"\n✅ Calibration report written to: {output_file.absolute()}")


if __name__ == "__main__":
    df = load_summary_data()
    generate_html_report(df)



from freemocap.data_layer.recording_models.post_processing_parameter_models import ProcessingParameterModel
from freemocap.data_layer.recording_models.recording_info_model import RecordingInfoModel


def processing_pipeline_check(processing_parameters: ProcessingParameterModel) -> None:
    """
    Performs checks on the processing parameters to make sure they are valid with the existing data before running the pipeline.
    Raises FileNotFoundError if any of the checks fail.
    """
    status_check_dict = processing_parameters.recording_info_model.status_check
    if not status_check_dict["synchronized_videos_status_check"]:
        raise FileNotFoundError(
            f"Could not find synchronized_videos folder at {processing_parameters.recording_info_model.synchronized_videos_folder_path}\n"
            "::: Synchronized videos are required to run the processing pipeline"
        )

    if not processing_parameters.tracking_parameters_model.run_image_tracking:
        if not status_check_dict["data2d_status_check"]:
            raise FileNotFoundError(
                f"No 2d data found at: {processing_parameters.recording_info_model.data_2d_npy_file_path}\n"
                "::: 2d data is required if you have turned off `Run 2d image tracking?` in the processing parameters"
            )

    if not processing_parameters.anipose_triangulate_3d_parameters_model.run_3d_triangulation:
        if not status_check_dict["data3d_status_check"] and not status_check_dict["single_video_check"]:
            raise FileNotFoundError(
                f"No 3d data found at: {processing_parameters.recording_info_model.data_3d_npy_file_path}\n"
                "::: 3d data is required if you have selected `Run 3d triangulation?` in the processing parameters"
                " and have more than 1 video in your synchronized videos folder"
            )
    else:
        if not status_check_dict["calibration_toml_check"] and not status_check_dict["single_video_check"]:
            raise FileNotFoundError(
                f"No calibration file found at: {processing_parameters.recording_info_model.calibration_toml_path}\n"
                ":::A calibration.toml file is required if you have selected `Run 3d triangulation?` in the processing parameters"
                " and have more than 1 video in your synchronized videos folder"
            )


if __name__ == "__main__":
    recording_info_model = RecordingInfoModel(
        recording_folder_path="/Users/philipqueen/freemocap_data/recording_sessions/freemocap_sample_data"
    )
    processing_parameters = ProcessingParameterModel(recording_info_model=recording_info_model)

    processing_pipeline_check(processing_parameters=processing_parameters)



import json
import logging
from pathlib import Path

import numpy as np

from freemocap.core_processes.capture_volume_calibration.anipose_camera_calibration import freemocap_anipose
from freemocap.core_processes.capture_volume_calibration.charuco_stuff.charuco_board_definition import charuco_7x5
from freemocap.core_processes.capture_volume_calibration.run_anipose_capture_volume_calibration import (
    run_anipose_capture_volume_calibration,
)
from freemocap.core_processes.capture_volume_calibration.triangulate_3d_data import triangulate_3d_data
from freemocap.data_layer.recording_models.recording_info_model import RecordingInfoModel
from freemocap.diagnostics.calibration.calibration_utils import get_charuco_2d_data
from freemocap.utilities.download_sample_data import download_test_data

# Configure logging
logger = logging.getLogger(__name__)


class SessionInfo:
    """
    Stores paths to key processed data files.
    """

    sample_session_folder_path: str
    recording_info_model: RecordingInfoModel


def setup_session():
    """
    Downloads sample data and processes it.
    Stores all important paths for easy access.
    """
    logger.info("Downloading sample data...")

    SessionInfo.sample_session_folder_path = download_test_data()

    logger.info("Initializing recording model...")
    SessionInfo.recording_info_model = RecordingInfoModel(
        recording_folder_path=SessionInfo.sample_session_folder_path,
        active_tracker="mediapipe",
    )

    logger.info("Calibrating")

    board_info_path = (
        Path(SessionInfo.sample_session_folder_path) / "charuco_board_info.json"
    )  # NOTE - I added this JSON to the sample data zip file to make it easier to get square size/height/width
    with open(board_info_path, "r", encoding="utf-8") as fh:
        board_info = json.load(fh)

    charuco_square_size = board_info["square_size_mm"]
    calibration_toml_path, _ = run_anipose_capture_volume_calibration(
        charuco_board_definition=charuco_7x5(),
        calibration_videos_folder_path=get_synchronized_video_folder_path(),
        charuco_square_size=charuco_square_size,
        progress_callback=lambda _: None,
    )

    charuco_2d_xy = get_charuco_2d_data(
        calibration_videos_folder_path=get_synchronized_video_folder_path(), num_processes=3
    )

    logger.info("Charuco 2d data detected successfully with shape: " f"{charuco_2d_xy.shape}")

    charuco_2d_xy = charuco_2d_xy.astype(np.float64)

    logger.info("Getting 3d Charuco data")
    anipose_calibration_object = freemocap_anipose.CameraGroup.load(str(calibration_toml_path))

    data_3d, *_ = triangulate_3d_data(
        anipose_calibration_object=anipose_calibration_object, image_2d_data=charuco_2d_xy
    )

    np.save(Path(SessionInfo.sample_session_folder_path) / "output_data" / "charuco_3d_xyz.npy", data_3d)

    logger.info("Session setup complete!")


def get_synchronized_video_folder_path():
    return Path(SessionInfo.recording_info_model.synchronized_videos_folder_path)


if __name__ == "__main__":
    setup_session()



import logging
from pathlib import Path

from pydantic import BaseModel
from src.config.home_dir import get_session_folder_path

logger = logging.getLogger(__name__)


class WebcamConfigModel(BaseModel):
    webcam_id: str
    exposure: int = -6
    resolution_width: int = 1280
    resolution_height: int = 720


def get_camera_name(camera_id: str):
    return "camera_" + camera_id


class UserConfigService:
    """take things like WebcamConfigModel and save it to a text file or whatever"""

    def save_webcam_config_to_disk(
            self, webcam_config_model: WebcamConfigModel, session_id: str
    ):
        webcam_config_json = webcam_config_model.json()
        camera_name = get_camera_name(webcam_config_model.webcam_id)
        json_file_name = camera_name + "_config.json"
        session_path = get_session_folder_path(session_id)
        webcam_config_json_path = Path(session_path) / json_file_name
        with open(webcam_config_json_path, "w") as outfile:
            outfile.write(str(webcam_config_json))
        return webcam_config_json

    def webcam_config_by_id(self, webcam_id: str, session_id: str) -> WebcamConfigModel:
        if session_id is None:
            return WebcamConfigModel(webcam_id=webcam_id)
        session_path = get_session_folder_path(session_id)
        camera_name = get_camera_name(webcam_id)
        print(f"webcam id (again): {webcam_id}")
        print(f"camera name: {camera_name}")
        json_file_name = camera_name + "_config.json"
        webcam_config_json_path = Path(session_path) / json_file_name

        try:
            webcam_config_model = WebcamConfigModel.parse_file(webcam_config_json_path)
            logger.info(f"loading webcam_config from: {webcam_config_json_path}")
            return webcam_config_model
        except FileNotFoundError:
            logger.info(
                f"No webcam config file found for this camera, using default camera parameters"
            )
            return WebcamConfigModel(webcam_id=webcam_id)



from typing import List, Dict, Optional

from freemocap.data_layer.skeleton_models.marker_info import MarkerInfo
from freemocap.data_layer.skeleton_models.segments import Segment, SegmentAnthropometry
from freemocap.data_layer.skeleton_models.skeleton import Skeleton


def create_marker_info(
        marker_list: List[str], virtual_markers: Optional[Dict[str, Dict[str, List]]] = None
) -> MarkerInfo:
    """
    Creates a MarkerInfo instance from a list of actual marker names and optional virtual markers.

    Parameters:
    - marker_list: A list of strings representing the names of actual markers.
    - virtual_markers: A dictionary defining virtual markers and their related data. Each key is a virtual marker name,
      and its value is another dictionary with 'marker_names' and 'marker_weights' as keys.

    Returns:
    - An instance of MarkerInfo populated with actual and, if provided, virtual markers.
    """
    marker_hub = MarkerInfo.create(marker_list=marker_list)

    if virtual_markers:
        marker_hub.add_virtual_markers(virtual_markers)
    return marker_hub


def create_skeleton_model(
        actual_markers: List[str],
        num_tracked_points: int,
        segment_connections: Optional[Dict[str, Segment]] = None,
        virtual_markers: Optional[Dict[str, Dict[str, List]]] = None,
        joint_hierarchy: Optional[Dict[str, List[str]]] = None,
        center_of_mass_info: Optional[Dict[str, SegmentAnthropometry]] = None,
) -> Skeleton:
    """
    Creates a Skeleton model that includes both actual and optionally virtual markers
    Optionally integrates segment connections, joint hierarchy, and anthro data if needed
    Parameters:
    - actual_markers: A list of strings representing the names of actual markers.
    - num_tracked_points: The number of tracked points expected in the 3D data.
    - segment_connections: Optional; A dictionary where each key is a segment name and its value is a dictionary
      with information about that segment (e.g., 'proximal', 'distal' marker names).
    - virtual_markers: Optional; a dictionary with information necessary to compute virtual markers.
    - joint_hierarchy: Optional; a dictionary with joint names as keys and lists of connected marker names as values.
    - center_of_mass_info: Optional; a dictionary containing segment mass percentages

    Returns:
    - An instance of the Skeleton class that represents the complete skeletal model including markers, segments,
      and optionally, joint hierarchy data.
    """
    marker_hub = create_marker_info(marker_list=actual_markers, virtual_markers=virtual_markers)

    skeleton_model = Skeleton(markers=marker_hub, num_tracked_points=num_tracked_points)

    if segment_connections:
        skeleton_model.add_segments(segment_connections)

    if joint_hierarchy:
        skeleton_model.add_joint_hierarchy(joint_hierarchy)

    if center_of_mass_info:
        skeleton_model.add_center_of_mass_definitions(center_of_mass_info)

    return skeleton_model



from fastapi import APIRouter

from src.config.data_paths import create_home_data_directory

startup_router = APIRouter()


@startup_router.on_event("startup")
async def handle_startup():
    create_home_data_directory()



"""A free and open source markerless motion capture system for everyone 💀✨"""

__author__ = """Skelly FreeMoCap"""
__email__ = "info@freemocap.org"
__version__ = "v1.7.4"
__description__ = "A free and open source markerless motion capture system for everyone 💀✨"

__package_name__ = "freemocap"
__repo_url__ = f"https://github.com/freemocap/{__package_name__}/"
__repo_issues_url__ = f"{__repo_url__}issues"

from freemocap.system.logging.configure_logging import configure_logging, LogLevel

configure_logging(LogLevel.TRACE)
import logging

logger = logging.getLogger(__name__)
logger.info(f"Initializing {__package_name__} package, version: {__version__}, from file: {__file__}")



from fastapi.middleware.cors import CORSMiddleware


def cors(app):
    app.add_middleware(
        CORSMiddleware,
        allow_origins=["*"],
        allow_credentials=True,
        allow_methods=["*"],
        allow_headers=["*"],
    )



import logging
import threading
from pathlib import Path

from PySide6.QtCore import Signal, QThread
from skelly_synchronize.skelly_synchronize import synchronize_videos_from_audio, synchronize_videos_from_brightness

logger = logging.getLogger(__name__)


class SynchronizeVideosThreadWorker(QThread):
    finished = Signal()
    in_progress = Signal(str)

    def __init__(
            self,
            raw_video_folder_path: Path,
            synchronized_video_folder_path: Path,
            kill_thread_event: threading.Event,
            synchronization_method: str = "audio",
            brightness_contrast_threshold: float = 1000,
    ):
        super().__init__()
        logger.info("Initializing Synchronize Videos Thread Worker")
        self._kill_thread_event = kill_thread_event
        self._raw_video_folder_path = raw_video_folder_path
        self._synchronized_video_folder_path = synchronized_video_folder_path
        self._synchronization_method = synchronization_method
        self._brightness_contrast_threshold = brightness_contrast_threshold

        self.output_folder_path = None

        self._work_done = False

    @property
    def work_done(self):
        return self._work_done

    def _emit_in_progress_data(self, message: str):
        self.in_progress.emit(message)

    def run(self):
        logger.info("Beginning to synchronize videos")

        try:
            if not self._synchronized_video_folder_path.exists():
                self._synchronized_video_folder_path.mkdir(parents=True, exist_ok=True)
            logger.info(f"Beginning to synchronize videos to folder {self._synchronized_video_folder_path}")
            if self._synchronization_method == "audio":
                self.output_folder_path = synchronize_videos_from_audio(
                    raw_video_folder_path=self._raw_video_folder_path,
                    synchronized_video_folder_path=self._synchronized_video_folder_path,
                    create_debug_plots_bool=False,
                )
            elif self._synchronization_method == "brightness":
                self.output_folder_path = synchronize_videos_from_brightness(
                    raw_video_folder_path=self._raw_video_folder_path,
                    synchronized_video_folder_path=self._synchronized_video_folder_path,
                    brightness_ratio_threshold=self._brightness_contrast_threshold,
                    create_debug_plots_bool=False,
                )
            else:
                raise Exception("Invalid synchronization method")

        except Exception as e:
            logger.exception("Something went wrong while synchronizing videos")
            logger.exception(e)

        self.finished.emit()
        self._work_done = True

        logger.info("Synchronizing Videos Complete")
